Package: SISIR
Type: Package
Title: Select Intervals Suited for Functional Regression
Version: 0.2.4
Date: 2026-05-14
Authors@R: c(
  person("Victor", "Picheny", , "victor.picheny@inrae.fr", c("aut"), 
    comment = c(ORCID = "0000-0002-4948-5542")),
  person("Remi", "Servien", , "remi.servien@inrae.fr", c("aut"), 
    comment = c(ORCID = "0000-0003-1270-1843")),
  person("Nathalie", "Vialaneix", , "nathalie.vialaneix@inrae.fr", 
         c("aut", "cre"), comment = c(ORCID = "0000-0003-1156-0639")))
Maintainer: Nathalie Vialaneix <nathalie.vialaneix@inrae.fr>
Description: Interval fusion and selection procedures for regression with 
             functional inputs. Methods include a semiparametric approach based
             on Sliced Inverse Regression (SIR), as described in 
             <doi:10.1007/s11222-018-9806-6> (standard ridge and sparse SIR are 
             also included in the package) and a random forest based approach, 
             as described in <doi:10.1002/sam.11705>.
Depends: R (>= 3.5.0), foreach, doParallel, graphics, stats
URL: https://sfcb.pages-forge.inrae.fr/sisir,
        https://forge.inrae.fr/sfcb/sisir
BugReports: https://forge.inrae.fr/sfcb/sisir/-/issues
Imports: Matrix, expm, RSpectra, glmnet, CORElearn, dplyr, mixOmics,
        purrr, ranger, tidyr, tidyselect, adjclust, magrittr, rlang,
        ggplot2, aricode, dendextend, reshape2, RColorBrewer, utils
Suggests: testthat, Boruta
License: GPL (>= 2)
Encoding: UTF-8
Repository: CRAN
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-14 15:41:37 UTC; nathalie
Author: Victor Picheny [aut] (ORCID: <https://orcid.org/0000-0002-4948-5542>),
  Remi Servien [aut] (ORCID: <https://orcid.org/0000-0003-1270-1843>),
  Nathalie Vialaneix [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1156-0639>)
Date/Publication: 2026-05-14 17:10:02 UTC
Built: R 4.6.0; ; 2026-05-14 19:27:36 UTC; unix

Package: SISIR
Type: Package
Title: Select Intervals Suited for Functional Regression
Version: 0.2.4
Date: 2026-05-14
Authors@R: c(
  person("Victor", "Picheny", , "victor.picheny@inrae.fr", c("aut"), 
    comment = c(ORCID = "0000-0002-4948-5542")),
  person("Remi", "Servien", , "remi.servien@inrae.fr", c("aut"), 
    comment = c(ORCID = "0000-0003-1270-1843")),
  person("Nathalie", "Vialaneix", , "nathalie.vialaneix@inrae.fr", 
         c("aut", "cre"), comment = c(ORCID = "0000-0003-1156-0639")))
Maintainer: Nathalie Vialaneix <nathalie.vialaneix@inrae.fr>
Description: Interval fusion and selection procedures for regression with 
             functional inputs. Methods include a semiparametric approach based
             on Sliced Inverse Regression (SIR), as described in 
             <doi:10.1007/s11222-018-9806-6> (standard ridge and sparse SIR are 
             also included in the package) and a random forest based approach, 
             as described in <doi:10.1002/sam.11705>.
Depends: R (>= 3.5.0), foreach, doParallel, graphics, stats
URL: https://sfcb.pages-forge.inrae.fr/sisir,
        https://forge.inrae.fr/sfcb/sisir
BugReports: https://forge.inrae.fr/sfcb/sisir/-/issues
Imports: Matrix, expm, RSpectra, glmnet, CORElearn, dplyr, mixOmics,
        purrr, ranger, tidyr, tidyselect, adjclust, magrittr, rlang,
        ggplot2, aricode, dendextend, reshape2, RColorBrewer, utils
Suggests: testthat, Boruta
License: GPL (>= 2)
Encoding: UTF-8
Repository: CRAN
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-14 15:41:37 UTC; nathalie
Author: Victor Picheny [aut] (ORCID: <https://orcid.org/0000-0002-4948-5542>),
  Remi Servien [aut] (ORCID: <https://orcid.org/0000-0003-1270-1843>),
  Nathalie Vialaneix [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1156-0639>)
Date/Publication: 2026-05-14 17:10:02 UTC
Built: R 4.6.0; ; 2026-05-14 18:59:26 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(extract_at,SFCB)
S3method(plot,SFCB)
S3method(print,SFCB)
S3method(print,SISIRres)
S3method(print,ridgeRes)
S3method(print,sparseRes)
S3method(project,sparseRes)
S3method(quality,SFCB)
S3method(summary,SFCB)
S3method(summary,SISIRres)
S3method(summary,ridgeRes)
S3method(summary,sparseRes)
export(SISIR)
export(extract_at)
export(project)
export(quality)
export(ridgeSIR)
export(sfcb)
export(sparseSIR)
export(tune.ridgeSIR)
import(dendextend)
import(doParallel)
import(foreach)
import(ggplot2)
import(glmnet)
import(magrittr)
importFrom(CORElearn,attrEval)
importFrom(Matrix,forceSymmetric)
importFrom(RColorBrewer,brewer.pal)
importFrom(RSpectra,eigs)
importFrom(adjclust,adjClust)
importFrom(adjclust,cutree_chac)
importFrom(aricode,AMI)
importFrom(aricode,ARI)
importFrom(aricode,NID)
importFrom(aricode,NMI)
importFrom(aricode,NVI)
importFrom(dplyr,arrange)
importFrom(dplyr,filter)
importFrom(dplyr,inner_join)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(expm,sqrtm)
importFrom(graphics,abline)
importFrom(mixOmics,pls)
importFrom(parallel,detectCores)
importFrom(purrr,map_dfc)
importFrom(ranger,ranger)
importFrom(reshape2,melt)
importFrom(rlang,.data)
importFrom(stats,as.dendrogram)
importFrom(stats,as.hclust)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,na.omit)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(tidyr,gather)
importFrom(tidyr,spread)
importFrom(tidyselect,starts_with)
importFrom(utils,head)
importFrom(utils,packageVersion)

SISIR
================

<!-- badges: start -->
[![CRAN version](https://www.r-pkg.org/badges/version/SISIR)](https://CRAN.R-project.org/package=SISIR)
[![CRAN checks](https://badges.cranchecks.info/summary/SISIR.svg)](https://cran.r-project.org/web/checks/check_results_SISIR.html)
[![CRANLOGS](https://cranlogs.r-pkg.org/badges/SISIR)](https://CRAN.R-project.org/package=SISIR)
[![SWH](https://archive.softwareheritage.org/badge/swh:1:dir:98f3e762fa7fbf55fb72e3f2a6bcc7c5a115e505/)](https://archive.softwareheritage.org/swh:1:dir:98f3e762fa7fbf55fb72e3f2a6bcc7c5a115e505;origin=https://forge.inrae.fr/sfcb/sisir.git;visit=swh:1:snp:c69201df54de7da3973c3104a31386c0cd856e08;anchor=swh:1:rev:8568af1877fec2324de0529df8dea7675961912a)
<!-- badges: end -->

`SISIR` is an **R** package designed to handle function data (e.g., curved
sampled on a discrete time grid). Implemented methods can perform interval
fusion and selection procedures in regression models with functional inputs.
They include a semiparametric approach based on Sliced Inverse Regression
(SIR), as described in
<a href="https://dx.doi.org/10.1007/s11222-018-9806-6">doi:10.1007/s11222-018-9806-6</a>
(standard ridge and sparse SIR are also included in the package) and a random
forest based approach, as described in <a href="https://dx.doi.org/10.1002/sam.11705">doi:10.1002/sam.11705</a>.

library(SISIR)

set.seed(1115)
tsteps <- seq(0, 1, length = 200)
nsim <- 100
simulate_bm <- function() return(c(0, cumsum(rnorm(length(tsteps)-1, sd=1))))
x <- t(replicate(nsim, simulate_bm()))
beta <- cbind(sin(tsteps*3*pi/2), sin(tsteps*5*pi/2))
y <- log(abs(x %*% beta[ ,1])) + sqrt(abs(x %*% beta[ ,2])) +
  rnorm(nsim, sd = 0.1)

# Tuning ----------------------------------------------------------------------
list_mu2 <- 10^(0:10)
listH <- c(5, 10)
list_d <- 1:4
set.seed(1129)
res_tune <- tune.ridgeSIR(x, y, listH, list_mu2, list_d, nfolds = 10, 
                          parallel = TRUE)

## for H=10
printable_res <- matrix(res_tune[res_tune$H == 10, "cverror"],
                        ncol=length(list_d), byrow = TRUE)
rownames(printable_res) <- log10(list_mu2)
colnames(printable_res) <- list_d
plot(log10(list_mu2), printable_res[ ,ncol(printable_res)], main = "",
     ylab = "CV error", xlab = expression(log[10](mu[2])), type = "b",
     pch = "+")
chosen_mu <- which.min(printable_res[ ,ncol(printable_res)])

printable_res2 <- matrix(res_tune[res_tune$H == 10, "Rd"],
                         ncol=length(list_d), byrow = TRUE)
rownames(printable_res2) <- log10(list_mu2)
colnames(printable_res2) <- list_d
plot(list_d, printable_res2[chosen_mu, ], type="b", pch="+", xlab = "d",
     ylab = expression(R(d)))
d <- 2

plot(log10(list_mu2), printable_res[ ,d], main = "",
     ylab = "CV error", xlab = expression(log[10](mu[2])), type = "b",
     pch = "+")
which.min(printable_res[ ,ncol(printable_res)]) # consistent

# Training --------------------------------------------------------------------
res_ridge <- ridgeSIR(x, y, H = 10, d = 2, mu2 = list_mu2[chosen_mu])
# > res_ridge
# Ridge SIR results with:
#   
#   10 slices
# dimension of the EDR space is: 2 
# regularization parameter is: 1e+08 
# The EDR space is in '$EDR'. First 10 rows are:
#   
#   10 x 2 Matrix of class "dgeMatrix"
# [,1]          [,2]
# [1,] 2.494088e-22  2.240430e-21
# [2,] 1.393174e-08  6.804754e-07
# [3,] 4.201512e-07  6.958648e-07
# [4,] 1.024709e-06  9.157455e-07
# [5,] 1.238248e-06  7.735075e-07
# [6,] 1.014715e-06  5.739618e-07
# [7,] 1.558069e-06 -6.855240e-07
# [8,] 1.752981e-06 -1.081445e-06
# [9,] 1.969968e-06 -2.109960e-06
# [10,] 2.580175e-06 -9.432574e-07

library("SISIR")

context("Testing ridge regression `ridgeSIR`")

test_that("Normalization in ridgeSIR works as expected.", {
  set.seed(1140)
  
  tsteps <- seq(0, 1, length = 200)
  nsim <- 100
  simulate_bm <- function() return(c(0, cumsum(rnorm(length(tsteps)-1, sd=1))))
  x <- t(replicate(nsim, simulate_bm()))
  beta <- cbind(sin(tsteps*3*pi/2), sin(tsteps*5*pi/2))
  y <- log(abs(x %*% beta[ ,1])) + sqrt(abs(x %*% beta[ ,2])) + 
    rnorm(nsim, sd = 0.1)
  
  # perform ridge regression
  res_ridge <- ridgeSIR(x, y, H = 10, d = 5, mu2 = 10^8)
  
  # test if normalization is OK
  norm_EDR <- res_ridge$utils$norm_EDR
  
  expect_equal(diag(res_ridge$parameters$d), 
               crossprod(res_ridge$EDR, norm_EDR) %*% res_ridge$EDR,
               tolerance = 10^(-10))
})

library(SISIR)

set.seed(1140)

tsteps <- seq(0, 1, length = 200)
nsim <- 100
simulate_bm <- function() return(c(0, cumsum(rnorm(length(tsteps)-1, sd=1))))
x <- t(replicate(nsim, simulate_bm()))
beta <- cbind(sin(tsteps*3*pi/2), sin(tsteps*5*pi/2))
beta[((tsteps < 0.2) || (tsteps > 0.5)), 1] <- 0
beta[((tsteps < 0.6) || (tsteps > 0.75)), 2] <- 0
y <- log(abs(x %*% beta[ ,1]) + 1) + sqrt(abs(x %*% beta[ ,2])) + 
  rnorm(nsim, sd = 0.1)

# perform ridge SIR
res_ridge <- ridgeSIR(x, y, H = 10, d = 2, mu2 = 10^5)
# perform sparse SIR
res_sparse <- sparseSIR(res_ridge, rep(1, ncol(x)))
print(res_sparse)
# interval fusion
res_fuse <- SISIR(res_ridge, rep(1, ncol(x)), parallel = TRUE)
print(res_fuse)
ind_best <- which.min(res_fuse$quality$CVerror)
cat("Best model for CV error has", res_fuse$quality$nbint[ind_best], 
    "intervals.", "\n\n")
res_best_fuse <- sparseSIR(res_ridge, res_fuse$intervals[[ind_best]])
print(res_best_fuse)

################################################################################
# sparse SIR
################################################################################
#' @import glmnet
#' @importFrom stats predict
#' 
#' @title sparse SIR
#' @export
#'
#' @description
#' \code{sparseSIR} performs the second step of the method (shrinkage of ridge
#' SIR results
#'
#' @param object an object of class \code{ridgeRes} as obtained from the 
#' function \code{\link{ridgeSIR}}
#' @param inter_len (numeric) vector with interval lengths
#' @param adaptive should the function returns the list of strong zeros and non
#' strong zeros (logical). Default to FALSE
#' @param sel_prop used only when \code{adaptive = TRUE}. Fraction of the 
#' coefficients that will be considered as strong zeros and strong non zeros.
#' Default to 0.05
#' @param parallel whether the computation should be performed in parallel or
#' not. Logical. Default is FALSE
#' @param ncores number of cores to use if \code{parallel = TRUE}. If left to 
#' NULL, all available cores minus one are used
#' 
#' @author {Victor Picheny, \email{victor.picheny@inrae.fr}\cr
#' Remi Servien, \email{remi.servien@inrae.fr}\cr
#' Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}}
#' 
#' @references {Picheny, V., Servien, R., and Villa-Vialaneix, N. (2019) 
#' Interpretable sparse SIR for digitized functional data. 
#' \emph{Statistics and Computing}, \strong{29}(2), 255--267.}
#' 
#' @seealso \code{\link{ridgeSIR}}, \code{\link{project.sparseRes}}, 
#' \code{\link{SISIR}}
#' 
#' @examples
#' set.seed(1140)
#' tsteps <- seq(0, 1, length = 200)
#' nsim <- 100
#' simulate_bm <- function() return(c(0, cumsum(rnorm(length(tsteps)-1, sd=1))))
#' x <- t(replicate(nsim, simulate_bm()))
#' beta <- cbind(sin(tsteps*3*pi/2), sin(tsteps*5*pi/2))
#' beta[((tsteps < 0.2) | (tsteps > 0.5)), 1] <- 0
#' beta[((tsteps < 0.6) | (tsteps > 0.75)), 2] <- 0
#' y <- log(abs(x %*% beta[ ,1]) + 1) + sqrt(abs(x %*% beta[ ,2]))
#' y <- y + rnorm(nsim, sd = 0.1)
#' res_ridge <- ridgeSIR(x, y, H = 10, d = 2, mu2 = 10^8)
#' res_sparse <- sparseSIR(res_ridge, rep(10, 20))
#' 
#' @return S3 object of class \code{sparseRes}: a list consisting of
#' \describe{
#'    \item{\code{sEDR}}{ the estimated EDR space (a p x d matrix)}
#'    \item{\code{alpha}}{ the estimated shrinkage coefficients (a vector having
#'    a length similar to \code{inter_len})}
#'    \item{\code{quality}}{ a vector with various qualities for the model (see
#'    Details)}
#'    \item{\code{adapt_res}}{ if \code{adaptive = TRUE}, a list of two vectors: 
#'    \describe{
#'      \item{\code{nonzeros}}{ indexes of variables that are strong non zeros}
#'      \item{\code{zeros}}{ indexes of variables that are strong zeros}
#'    }}
#'    \item{\code{parameters}}{ a list of hyper-parameters for the method: 
#'    \describe{
#'      \item{\code{inter_len}}{ lengths of intervals}
#'      \item{\code{sel_prop}}{ if \code{adaptive = TRUE}, fraction of the 
#'      coefficients which are considered as strong zeros or strong non zeros}
#'    }}
#'    \item{\code{rSIR}}{ same as the input \code{object}}
#'    \item{\code{fit}}{ a list for LASSO fit with:
#'    \describe{
#'      \item{\code{glmnet}}{ result of the \code{\link[glmnet]{glmnet}} 
#'      function}
#'      \item{\code{lambda}}{ value of the best Lasso parameter by CV}
#'      \item{\code{x}}{ exploratory variable values as passed to fit the 
#'      model}
#'    }}
#'  }
#'  
#'  @details Different quality criteria used to select the best models among a
#'  list of models with different interval definitions. Quality criteria are:
#'  log-likelihood (\code{loglik}), cross-validation error as provided by the
#'  function \code{\link[glmnet]{glmnet}}, two versions of the AIC (\code{AIC} 
#'  and \code{AIC2}) and of the BIC (\code{BIC} and \code{BIC2}) in which the 
#'  number of parameters is either the number of non null intervals or the 
#'  number of non null parameters with respect to the original variables.

sparseSIR <- function(object, inter_len, adaptive = FALSE, sel_prop = 0.05,
                      parallel = FALSE, ncores = NULL) {
  if (parallel) {
    if (is.null(ncores)) ncores <- min(detectCores() - 1)
    print(ncores)
    registerDoParallel(cores = ncores)
  }
  oldwarn <- getOption("warn")
  options(warn = -1)
  
  H <- object$parameters$H
  D <- length(inter_len)
  p <- nrow(object$EDR)
  if (sum(inter_len) != p)
    stop("Sum of 'inter_len' must be equal to the number of columns in original dataset.")
  d <- ncol(object$EDR)
  cmean_x <- object$utils$cmean_x
  slices <- object$utils$slices
  rEDR <- object$EDR
  x <- object$data$x
  
  coef_elastic <- 1 # note: can be passed as an argument for elastinet
  
  # definition of the dependant variable: vector of length dn
  Proj <- crossprod(sweep(cmean_x[ ,slices], 1, apply(x, 2, mean), "-") , rEDR)
  Proj <- as.vector(Proj)
  
  # definition of the explanatory variables: matrix of dimension (dn)xD
  explanatory <- apply(rEDR, 2, function(acol) {
    Delta <- matrix(0, p, D)
    Delta[cbind(1:p, rep(1:D, inter_len))] <- acol
    return(list("out" = x %*% Delta))
  })
  explanatory <- lapply(explanatory, function(alist) alist$out)
  explanatory <- Reduce(rbind, explanatory)
  
  # model fit (CV)
  fit <- cv.glmnet(x = explanatory, y = Proj, alpha = coef_elastic, 
                   parallel = parallel)
  if (parallel) {
    stopImplicitCluster()
  }
  if (length(fit$lambda) < 20) {# in this case, fit on a very fine grid
    lambda_seq <- fit$lambda[1] * 10^(-seq(1, 10, length = 1000))
    fit <- cv.glmnet(x = explanatory, y = Proj, alpha = coef_elastic, 
                     lambda = lambda_seq, parallel = parallel)
    if (parallel) {
      stopImplicitCluster()
    }
  }
  best_lambda <- fit$lambda.min
  ind_best <- which(best_lambda == fit$lambda.min)
  
  # re-fit with a finer grid to obtain the whole regularization path (no CV)
  seq_lambda <- c(as.vector(outer(seq(1:990)/100, 10**(1:-10), "*")),
                  best_lambda)
  seq_lambda <- sort(unique(seq_lambda), decreasing = TRUE)
  ind_best <- which(seq_lambda == best_lambda)
  
  best_fit <- glmnet(x = explanatory, y = Proj, alpha = coef_elastic,
                     lambda = seq_lambda, family="gaussian")
  alpha <- best_fit$beta[ ,ind_best]

  sEDR <- sweep(rEDR, 1, rep(alpha, inter_len), "*")
  
  # quality criteria TODO: see which ones to keep
  nobs <- fit$glmnet.fit$nobs
  loglik <- log(mean((predict(fit, explanatory, s=best_lambda) - Proj)^2))
  AIC <- nobs * loglik + 2 * sum(alpha != 0)
  BIC <- nobs * loglik + sum(alpha != 0) * log(nobs)
  nb_param <- sum(as.numeric(alpha != 0) * inter_len)
  AIC2 <- nobs * loglik + 2 * nb_param
  BIC2 <- nobs * loglik + nb_param * log(nobs)
  CVerror <- min(fit$cvm)
  quality <- c(loglik, AIC, BIC, AIC2, BIC2, CVerror)
  names(quality) <- c("loglik", "AIC", "BIC", "AIC2", "BIC2", "CVerror")
  
  if (adaptive) {# find hard/soft zeros
    ind_nonzero <- which.min(abs(best_fit$df - sel_prop*D))
    alpha_nonzero <- best_fit$beta[ ,ind_nonzero]
    ind_zero <- which.min(abs(best_fit$df - (1-sel_prop)*D))
    alpha_zero <- best_fit$beta[ ,ind_zero]
    adapt <- list("nonzeros" = alpha_nonzero, "zeros" = alpha_zero)
  } else adapt <- NULL
  
  parameters <- list("inter_len" = inter_len, "sel_prop" = sel_prop)

  res <- list("sEDR" = sEDR, "alpha" = unname(alpha), "quality" = quality, 
              "adapt_res" = adapt, "parameters" = parameters, "rSIR" = object,
              "fit" = list("glmnet" = best_fit, "lambda" = best_lambda,
                           "x" = explanatory))
  class(res) <- "sparseRes"
  
  options(warn = oldwarn)
  
  return(res)
}

################################################################################
# Methods for objects of class sparseRes
################################################################################
#' @title Print sparseRes object
#' @name sparseRes
#' @export
#' @aliases summary.sparseRes
#' @aliases summary.sparseRes
#' @aliases print.sparseRes
#' @aliases sparseRes-class
#' @description Print a summary of the result of \code{\link{sparseSIR}} (
#' \code{sparseRes} object)
#' @param object a \code{sparseRes} object
#' @param x a \code{sparseRes} object
#' @param ... not used
#' @author {Victor Picheny, \email{victor.picheny@inrae.fr}\cr
#' Remi Servien, \email{remi.servien@inrae.fr}\cr
#' Nathalie Vialaneix, \email{nathalie.vialaneix@inra.fr}}
#' @seealso \code{\link{sparseSIR}}
#' 
summary.sparseRes <- function(object, ...) {
  cat("Sparse SIR results with:\n\n",
      object$rSIR$parameters$H, "slices\n",
      "dimension of the EDR space is:", object$rSIR$parameters$d, "\n",
      "regularization parameter is:", object$rSIR$parameters$mu2, "\n\n")
  cat(sum(object$alpha != 0), "non zero coefficient(s) out of", 
      length(object$alpha), "possible.\n")
  cat("Shrinkage coefficients are in $alpha.", min(c(length(object$alpha), 10)),
      "first coefficients are:\n")
  print(head(object$alpha, 10))
  cat("\nThe EDR space is in '$sEDR'. First 10 rows are:\n\n")
  print(head(object$sEDR, 10))
}

#' @export
#' @rdname sparseRes
print.sparseRes <- function(x, ...) {
  summary.sparseRes(x)
}

#' @title sparse SIR
#' @name project
#' @export
#' @aliases project.sparseRes
#'
#' @description
#' \code{project} performs the projection on the sparse EDR space (as obtained
#' by the \code{\link[glmnet]{glmnet}})
#'
#' @param object an object of class \code{sparseRes} as obtained from the 
#' function \code{\link{sparseSIR}}
#' 
#' @author {Victor Picheny, \email{victor.picheny@inrae.fr}\cr
#' Remi Servien, \email{remi.servien@inrae.fr}\cr
#' Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}}
#' 
#' @references {Picheny, V., Servien, R. and Villa-Vialaneix, N. (2016) 
#' Interpretable sparse SIR for digitized functional data.
#' \emph{Statistics and Computing}, \strong{29}(2), 255--267.}
#' 
#' @seealso \code{\link{sparseSIR}}
#' 
#' @details The projection is obtained by the function 
#' \code{\link[glmnet]{predict.glmnet}}.
#' 
#' @examples
#' set.seed(1140)
#' tsteps <- seq(0, 1, length = 200)
#' nsim <- 100
#' simulate_bm <- function() return(c(0, cumsum(rnorm(length(tsteps)-1, sd=1))))
#' x <- t(replicate(nsim, simulate_bm()))
#' beta <- cbind(sin(tsteps*3*pi/2), sin(tsteps*5*pi/2))
#' beta[((tsteps < 0.2) | (tsteps > 0.5)), 1] <- 0
#' beta[((tsteps < 0.6) | (tsteps > 0.75)), 2] <- 0
#' y <- log(abs(x %*% beta[ ,1]) + 1) + sqrt(abs(x %*% beta[ ,2]))
#' y <- y + rnorm(nsim, sd = 0.1)
#' \donttest{
#' res_ridge <- ridgeSIR(x, y, H = 10, d = 2)
#' res_sparse <- sparseSIR(res_ridge, rep(1, ncol(x)))
#' proj_data <- project(res_sparse)
#' }
#' 
#' @return a matrix of dimension n x d with the projection of the observations
#' on the d dimensions of the sparse EDR space
#' 
project.sparseRes <- function(object) {
  res <- predict.glmnet(object$fit$glmnet, object$fit$x, object$fit$lambda)
  res <- matrix(res, ncol = object$rSIR$parameters$d, byrow = FALSE)
  return(res)
}

#' @export
#' @rdname project
project <- function(object) {
  UseMethod("project")
}

################################################################################
# Iterative interval search
################################################################################
#' @title Interval Sparse SIR
#' @export
#'
#' @description
#' \code{SISIR} performs an automatic search of relevant intervals
#'
#' @param object an object of class \code{ridgeRes} as obtained from the 
#' function \code{\link{ridgeSIR}}
#' @param inter_len (numeric) vector with interval lengths for the initial 
#' state. Default is to set one interval for each variable (all intervals have
#' length 1)
#' @param sel_prop fraction of the coefficients that will be considered as 
#' strong zeros and strong non zeros. Default to 0.05
#' @param itermax maximum number of iterations. Default to Inf
#' @param minint minimum number of intervals. Default to 2
#' @param parallel whether the computation should be performed in parallel or
#' not. Logical. Default is FALSE
#' @param ncores number of cores to use if \code{parallel = TRUE}. If left to 
#' NULL, all available cores minus one are used
#'  
#' @details 
#' Different quality criteria used to select the best models among a list of 
#' models with different interval definitions. Quality criteria are: 
#' log-likelihood (\code{loglik}), cross-validation error as provided by the
#' function \code{\link[glmnet]{glmnet}}, two versions of the AIC (\code{AIC} 
#' and \code{AIC2}) and of the BIC (\code{BIC} and \code{BIC2}) in which the 
#' number of parameters is either the number of non null intervals or the 
#' number of non null parameters with respect to the original variables
#' 
#' @author Victor Picheny, \email{victor.picheny@inrae.fr}\cr
#' Remi Servien, \email{remi.servien@inrae.fr}\cr
#' Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}
#' 
#' @references Picheny, V., Servien, R. and Villa-Vialaneix, N. (2016) 
#' Interpretable sparse SIR for digitized functional data.
#' \emph{Statistics and Computing}, \strong{29}(2), 255--267.
#' 
#' @seealso \code{\link{ridgeSIR}}, \code{\link{sparseSIR}}
#' 
#' @examples
#' set.seed(1140)
#' tsteps <- seq(0, 1, length = 200)
#' nsim <- 100
#' simulate_bm <- function() return(c(0, cumsum(rnorm(length(tsteps)-1, sd=1))))
#' x <- t(replicate(nsim, simulate_bm()))
#' beta <- cbind(sin(tsteps*3*pi/2), sin(tsteps*5*pi/2))
#' beta[((tsteps < 0.2) | (tsteps > 0.5)), 1] <- 0
#' beta[((tsteps < 0.6) | (tsteps > 0.75)), 2] <- 0
#' y <- log(abs(x %*% beta[ ,1]) + 1) + sqrt(abs(x %*% beta[ ,2]))
#' y <- y + rnorm(nsim, sd = 0.1)
#' res_ridge <- ridgeSIR(x, y, H = 10, d = 2, mu2 = 10^8)
#' \donttest{res_fused <- SISIR(res_ridge, rep(1, ncol(x)), ncores = 2)
#' res_fused}
#' 
#' @return S3 object of class \code{SISIR}: a list consisting of
#' \describe{
#'    \item{\code{sEDR}}{ the estimated EDR spaces (a list of p x d matrices)}
#'    \item{\code{alpha}}{ the estimated shrinkage coefficients (a list of 
#'    vectors)}
#'    \item{\code{intervals}}{ the interval lengths (a list of vectors)}
#'    \item{\code{quality}}{ a data frame with various qualities for the model.
#'    The chosen quality measures are the same than for the function 
#'    \code{\link{sparseSIR}} plus the number of intervals \code{nbint}}
#'    \item{\code{init_sel_prop}}{ initial fraction of the coefficients which 
#'    are considered as strong zeros or strong non zeros}
#'    \item{\code{rSIR}}{ same as the input \code{object}}
#'  }
#'  
SISIR <- function(object, inter_len = rep(1, nrow(object$EDR)), sel_prop = 0.05,
                  itermax = Inf, minint = 2, parallel = TRUE, ncores = NULL) {
  
  init_sel_prop <- sel_prop
  
  nbiter <- 1
  loglik <- AIC <- BIC <- AIC2 <- BIC2 <- CVerror <- NULL
  sEDR <- list()
  alpha <- list()
  intervals <- list()
  intervals[[1]] <- inter_len
  while ((sel_prop < 0.49) & (nbiter < itermax) & length(inter_len) > minint) {
    sparse_res <- sparseSIR(object, inter_len, adaptive = TRUE, sel_prop,
                            parallel, ncores)
    
    # collect results
    CVerror <- c(CVerror, sparse_res$quality["CVerror"])
    AIC <- c(AIC, sparse_res$quality["AIC"])
    BIC <- c(BIC, sparse_res$quality["BIC"])
    AIC2 <- c(AIC2, sparse_res$quality["AIC2"])
    BIC2 <- c(BIC2, sparse_res$quality["BIC2"])
    loglik <- c(loglik, sparse_res$quality["loglik"])

    sEDR[[nbiter]] <- sparse_res$sEDR
    alpha[[nbiter]] <- sparse_res$alpha
    
    ## merge intervals (sequential from left to right)
    liste_strong0 <- which(sparse_res$adapt_res$nonzeros != 0)
    liste_strongnot0 <- which(sparse_res$adapt_res$zeros == 0)
    df_merge <- data.frame("int" = c(liste_strong0, liste_strongnot0),
                           "type" = c(rep("yes", length(liste_strong0)),
                                      rep("no", length(liste_strongnot0))))
    df_merge <- df_merge[order(df_merge$int), ]
    nb_tests <- nrow(df_merge)
    
    # merging at left border
    if (df_merge$int[1] > 1) {
      leftbd <- sum(inter_len[1:(df_merge$int[1] - 1)])
      if (leftbd < inter_len[df_merge$int[1]]) {
        inter_len[df_merge$int[1]] <- inter_len[df_merge$int[1]] + leftbd
        inter_len[1:(df_merge$int[1] - 1)] <- 0
      }
    }
    
    # merge between intervals of same types
    if (nb_tests > 1) {
      may_merge <- df_merge$type[1:(nb_tests-1)] == df_merge$type[2:nb_tests]
    } else may_merge <- FALSE
    if (sum(may_merge) > 0) {
      # merge if two intervals are consecutive
      merge1 <- (df_merge$int[1:(nb_tests-1)] + 1) == df_merge$int[2:nb_tests]
      # merge if around length is larger than between length
      around_len <- inter_len[df_merge$int[1:(nb_tests-1)]] +
        inter_len[df_merge$int[2:nb_tests]]
      between_len <- sapply(1:(nb_tests-1), function(pos) {
        sum(inter_len[(df_merge$int[pos] + 1):(df_merge$int[pos+1] - 1)])
      })
      merge2 <- around_len > between_len
      # conclusion and merge
      merge <- which(may_merge & (merge1 | merge2))
      
      # sequential merge (from left to right)
      for (pos in merge) {
        m_length <- sum(inter_len[df_merge$int[pos]:df_merge$int[pos + 1]])
        inter_len[df_merge$int[pos + 1]] <- m_length
       inter_len[df_merge$int[pos]:(df_merge$int[pos + 1] - 1)] <- 0
      }
    }
    
    # right border
    if (df_merge$int[nb_tests] < length(inter_len)) {
      rightbd <- sum(inter_len[(df_merge$int[nb_tests] + 1):length(inter_len)])
      if (rightbd < inter_len[df_merge$int[nb_tests]]) {
        inter_len[df_merge$int[nb_tests]] <- inter_len[df_merge$int[nb_tests]] +
          rightbd
        inter_len[(df_merge$int[nb_tests] + 1):length(inter_len)] <- 0
      }
    }
    
    # final intervals
    inter_len <- inter_len[inter_len != 0]
    intervals[[nbiter + 1]] <- inter_len
    cat("Current number of intervals...", length(inter_len), "\n")
    
    if (length(intervals[[nbiter + 1]]) == length(intervals[[nbiter]])) 
      sel_prop <- sel_prop + 0.01

    nbiter <- nbiter + 1
  }
  
  intervals[[nbiter]] <- NULL
  quality <- data.frame("loglik" = loglik, "AIC" = AIC, "AIC2" = AIC2, 
                        "BIC" = BIC, "BIC2" = BIC2, "CVerror" = CVerror,
                        "nbint" = unlist(lapply(intervals, length)))
  res <- list("sEDR" = sEDR, "alpha" = alpha, "intervals" = intervals,
              "quality" = quality, "init_sel_prop" = init_sel_prop, 
              "rSIR" = object)
  class(res) <- "SISIRres"
  return(res)
}

################################################################################
# Methods for objects of class SISIRres
################################################################################
#' @title Print SISIRres object
#' @name SISIRres
#' @export
#' @aliases summary.SISIRres
#' @aliases print.SISIRres
#' @description Print a summary of the result of \code{\link{SISIRres}} (
#' \code{SISIRres} object)
#' @param object a \code{SISIRres} object
#' @param x a \code{SISIRres} object
#' @param ... not used
#' @author {Victor Picheny, \email{victor.picheny@inrae.fr}\cr
#' Remi Servien, \email{remi.servien@inrae.fr}\cr
#' Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}}
#' @seealso \code{\link{SISIR}}
#' 
summary.SISIRres <- function(object, ...) {
  cat("Interval Sparse SIR results with:\n\n",
      object$rSIR$parameters$H, "slices\n",
      "dimension of the EDR space is:", object$rSIR$parameters$d, "\n",
      "regularization parameter is:", object$rSIR$parameters$mu2, "\n\n")
  cat("Number of fitted models:", length(object$sEDR), "\n\n")
}

#' @export
#' @rdname SISIRres
print.SISIRres <- function(x, ...) {
  summary.SISIRres(x)
}

# related to group selection ####

summary2original <- function(selected, groups, varnames) {
  input_selected <- varnames[groups %in% selected]
  return(input_selected)
}

summary2originalb <- function(selected, groups, summaries, varnames) {
  input_selected <- names(summaries)[selected]
  mean_selected <- grep("mean", input_selected, value = TRUE)
  mean_selected <- as.numeric(gsub("mean\\.g", "", mean_selected))
  sd_selected <- grep("sd", input_selected, value = TRUE)
  sd_selected <- as.numeric(gsub("sd\\.g", "", sd_selected))
  input_selected <- varnames[groups %in% unique(c(mean_selected, sd_selected))]
  in_mean <- varnames[groups %in% mean_selected]
  in_sd <- varnames[groups %in% sd_selected]
  input_selected <- data.frame("variable" = input_selected, 
                               "in.mean"  = input_selected %in% in_mean,
                               "in.sd"    = input_selected %in% in_sd)
  return(input_selected)
}

# related to random forest fitting ####
#' @importFrom ranger ranger

predict_rf <- function(summaries, target, input_groups, selection = "none", 
                       repeats, seed, is_basics = FALSE, varnames) {
  # Description #####
  # returns a list with information on qualities of RF (importance, MSE) for the
  # given summary list and groups to predict a target
  # Inputs #####
  # summaries: a list with a hierarchy of summaries
  # target: the variable to predict
  # input_groups: a list with a hierarchy of groups corresponding to summaries
  # selection (optional): a list with selected features at each level of the
  # hierarchy
  # repeats: number of times the RF is fitted
  # seed: random seed
  # is_basics: are the simulations based on basics predictions
  # varnames: names of input variables
  # Outputs #####
  # a list with two entries: 1/ qualities is a data.frame where rows correspond
  # to experiments (5 for each level of the hierarchy) and columns contain 
  # MSE, number of clusters and (optionally) mean importance for truly 
  # important and non important variables as well as p-value of the Wilcoxon
  # test comparing the distribution of importances in the two groups of 
  # variables; 2/ importance is a list where each entry corresponds to an 
  # experiment and contains importance of the original variables as well as 
  # (optionally) importance of the truly important and non important variables
  
  summary_names <- names(summaries)
  
  if (length(selection) > 1 || selection != "none") {
    summaries <- lapply(1:length(summaries), function(ind) {
      out <- summaries[[ind]][, selection[[ind]], drop = FALSE]
      colnames(out) <- colnames(summaries[[ind]])[selection[[ind]]]
      return(out)
    })
  }
  
  ## compute random forests
  quality_criteria <- lapply(summaries, compute_rfs, target = target, 
                             repeats = repeats, seed = seed)
  quality_criteria <- Reduce(c, quality_criteria)
  
  ## MSE
  all_mse <- sapply(quality_criteria, function(alist) alist$mse)
  all_mse <- data.frame("clust" = rep(summary_names, each = repeats),
                        "mse" = all_mse)
  all_mse$clust <- as.numeric(all_mse$clust)
  
  ## extracting importance
  all_importances <- sapply(quality_criteria, function(alist) alist$importance,
                            simplify = FALSE)
  freq_groups <- sapply(input_groups, table, simplify = FALSE)
  orig_importance <- lapply(1:length(all_importances), compute_origimportance,
                            importances  = all_importances, 
                            input_groups = input_groups, 
                            freq_groups  = freq_groups,
                            varnames     = varnames,
                            repeats      = repeats, 
                            is_basics    = is_basics)
  orig_importance <- sapply(1:length(summaries), function(ind) {
    out <- Reduce(cbind, orig_importance[(repeats*(ind-1)+1):(repeats*ind)])
    if (!is_basics) {
      colnames(out) <- paste0("rep", 1:repeats)
    } else {
      colnames(out) <- paste0(rep(c("mean", "sd"), repeats), 
                              rep(1:repeats, each = 2))
    }
    return(out)
  }, simplify = FALSE)
  names(orig_importance) <- summary_names
  
  out <- list("mse" = all_mse, "importances" = orig_importance)
  
  return(out)
}

compute_rfs <- function(summaries, target, repeats, seed) {
  if (!any(is.na(summaries)) && ncol(summaries) > 0) {
    
    curdf <- data.frame(y = target, summaries)
    
    set.seed(seed)
    random_seeds <- sample(1:10^6, repeats, replace = FALSE)
    cur_model <- sapply(1:repeats, function(ind) {
      ranger::ranger(y ~ ., data = curdf, importance = "permutation", 
                     seed = random_seeds[ind], num.threads = 6)
    }, simplify = FALSE)
    out <- lapply(cur_model, function(ares) {
      list("mse" = ares$prediction.error,
           "importance" = ares$variable.importance)
    })
    
  } else {
    
    out <- lapply(1:repeats, function(ind) 
      list("mse" = NA, "importance" = rep(NA, ncol(summaries))))
  
  }
  
  return(out)
}

compute_origimportance <- function(ind, importances, input_groups, freq_groups,
                                   varnames, repeats, is_basics) {
  expe <- (ind - 1)%/%repeats + 1
    
  if (!is_basics) {
    
    groups_in_imp <- as.numeric(gsub("g", "", names(importances[[ind]])))
    frequencies <- freq_groups[[expe]][as.character(groups_in_imp)]
    out <- rep(importances[[ind]], frequencies)
    names(out) <- varnames[input_groups[[expe]] %in% groups_in_imp]
  
  } else {
    
    sel_in_means <- grep("mean", names(importances[[ind]]), value = TRUE)
    groups_in_means <-  gsub("mean\\.g", "", sel_in_means)
    sel_in_sd <- grep("sd", names(importances[[ind]]), value = TRUE)
    groups_in_sd <-  gsub("sd\\.g", "", sel_in_sd)
    freq_means <- freq_groups[[expe]][groups_in_means]
    freq_sd <- freq_groups[[expe]][groups_in_sd]
    out_means <- rep(importances[[ind]][sel_in_means], freq_means)
    names_means <- varnames[input_groups[[expe]] %in% groups_in_means]
    out_sd <- rep(importances[[ind]][sel_in_sd], freq_sd)
    names_sd <- varnames[input_groups[[expe]] %in% groups_in_sd]
    out <- merge(data.frame("var" = names_means, "mean" = out_means), 
                 data.frame("var" = names_sd, "sd" = out_sd),
                 by = "var", all = TRUE)
    rownames(out) <- out$var
    out <- out[varnames, ]
    out$var <- NULL
    
  }
  
  return(out)
}

#' SI-SIR
#'
################################################################################
# ridge SIR
################################################################################
#' @import foreach
#' @import doParallel
#' @importFrom Matrix forceSymmetric
#' @importFrom stats cov
#' @importFrom stats quantile
#' @importFrom utils head
#' @importFrom parallel detectCores
#' @importFrom RSpectra eigs
#' @importFrom expm sqrtm
#' 
#' @title ridge SIR
#' @export
#'
#' @description
#' \code{ridgeSIR} performs the first step of the method (ridge regularization
#' of SIR)
#'
#' @param x explanatory variables (numeric matrix or data frame)
#' @param y target variable (numeric vector)
#' @param H number of slices (integer)
#' @param mu2 ridge regularization parameter (numeric, positive)
#' @param d number of dimensions to be kept
#' 
#' @author Victor Picheny, \email{victor.picheny@inrae.fr}\cr
#' Remi Servien, \email{remi.servien@inrae.fr}\cr
#' Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}
#' 
#' @references Picheny, V., Servien, R. and Villa-Vialaneix, N. (2019) 
#' Interpretable sparse SIR for digitized functional data. 
#' \emph{Statistics and Computing}, \strong{29}(2), 255--267.
#' 
#' @seealso \code{\link{sparseSIR}}, \code{\link{SISIR}}, 
#' \code{\link{tune.ridgeSIR}}
#' 
#' @examples
#' set.seed(1140)
#' tsteps <- seq(0, 1, length = 50)
#' simulate_bm <- function() return(c(0, cumsum(rnorm(length(tsteps)-1, sd=1))))
#' x <- t(replicate(50, simulate_bm()))
#' beta <- cbind(sin(tsteps*3*pi/2), sin(tsteps*5*pi/2)) 
#' y <- log(abs(x %*% beta[ ,1])) + sqrt(abs(x %*% beta[ ,2]))
#' y <- y + rnorm(50, sd = 0.1)
#' res_ridge <- ridgeSIR(x, y, H = 10, d = 2, mu2 = 10^8)
#' res_ridge
#' 
#' @return S3 object of class \code{ridgeRes}: a list consisting of
#' \describe{
#'    \item{\code{EDR}}{ the estimated EDR space (a p x d matrix)}
#'    \item{\code{condC}}{ the estimated slice projection on EDR (a d x H 
#'    matrix)}
#'    \item{\code{eigenvalues}}{ the eigenvalues obtained during the generalized 
#'    eigendecomposition performed by SIR}
#'    \item{\code{parameters}}{ a list of hyper-parameters for the method: 
#'    \describe{
#'      \item{\code{H}}{ number of slices}
#'      \item{\code{d}}{ dimension of the EDR space}
#'      \item{\code{mu2}}{ regularization parameter for the ridge penalty}
#'    }}
#'    \item{\code{utils}}{ useful outputs for further computations:
#'    \describe{
#'      \item{\code{Sigma}}{ covariance matrix for x}
#'      \item{\code{slices}}{ slice number for all observations}
#'      \item{\code{invsqrtS}}{ value of the inverse square root of the 
#'      regularized covariance matrix for x}
#'    }}
#'  }
#'  

ridgeSIR <- function(x, y, H, d, mu2 = NULL) {
  if (is.data.frame(x)) x <- as.matrix(x)
  
  oldwarn <- getOption("warn")
  options(warn = -1)
  
  # precomputations
  ## for slices
  slices <- makeSlices(y, H)
  
  ## for x  
  pre <- preSIR(x, slices, H)
  
  ## for mu2
  suppressWarnings({
    smallest <- eigs(pre$cov_x, 1, which = "SR")$values
  })

  if (is.null(mu2)) {
    if (is.null(smallest)) {
      mu2 <- 10^(-5)
    } else if (smallest == Inf) {
      mu2 <- 10^(-5)
    } else if (smallest > 10^(-5)) {
      mu2 <- 0
    } else if (smallest < 0) {
      mu2 <- - smallest * 10^8
    } else mu2 <- 10^(-5)
  } else {
    if (length(smallest) > 0) {
      if ((smallest < 0) && (mu2 < 10^8 * abs(smallest))) {
        warning(paste("\n\nThe value provided for mu2 might lead to numerical",
                      "instabilities.\nMinimum advised value for mu2 is", 
                       smallest * 10^8))
      }
    }
  }
  
  # processing SIR
  resSIR <- processSIR(pre, d, mu2)
  
  # output
  precomputations <- list("Sigma" = pre$cov_x, "slices" = slices$slices,
                          "cmean_x" = pre$cmean_x, "norm_EDR" = resSIR$norm_EDR,
                          "invsqrtS" = resSIR$invsqrtS)
  orig_data <- list("x" = x, "y" = y)
  parameters <- list("H" = H, "d" = d, "mu2" = mu2)
  
  res <- list("EDR" = resSIR$EDR, "condC" = resSIR$condC,
              "eigenvalues" = resSIR$eigenvalues, "data" = orig_data,
              "parameters" = parameters, "utils" = precomputations)
  class(res) <- "ridgeRes"
  
  options(warn = oldwarn)
  return(res)
}

makeSlices <- function(y, H) {
  slices_bounds <- quantile(y, probs=seq(0, 1, length = H+1))
  slices_y <- cut(y, breaks=slices_bounds, labels=FALSE, include.lowest=TRUE)
  freq_slices <- as.vector(table(slices_y)) / length(y)
  return(list("slices" = slices_y, "freq" = freq_slices))
}

preSIR <- function(x, slices, H) {
  p <- ncol(x)
  mean_x <- colMeans(x)
  cov_x <- cov(x)
  
  ## conditional mean and variance
  cmean_x <- split(as.data.frame(x), slices$slices)
  cmean_x <- matrix(unlist(lapply(cmean_x, colMeans)), ncol = H)
  cmean_x <- cmean_x - mean_x
  ccov_x <- cmean_x %*% tcrossprod(diag(slices$freq), cmean_x)
  
  return(list("cmean_x" = cmean_x, "cov_x" = cov_x, "ccov_x" = ccov_x))
}

processSIR <- function(pre, d, mu2) {
  # computing inverse and square root of matrices
  norm_EDR <- pre$cov_x + mu2 * diag(ncol(pre$cov_x))
  suppressWarnings({sqrtS <- forceSymmetric(sqrtm(norm_EDR))})
  invsqrtS <- solve(sqrtS)
  
  # EDR
  eta <- forceSymmetric(invsqrtS %*% pre$ccov_x %*% invsqrtS)
  res <- eigs(eta, d)
  EDR <- invsqrtS %*% res$vectors
  eig_vals <- res$values
  
  # C
  cC <- crossprod(EDR, pre$cmean_x)
  
  return(list("EDR" = EDR, "condC" = cC, "eigenvalues" = eig_vals, 
              "norm_EDR" = norm_EDR, "invsqrtS" = invsqrtS))
}

################################################################################
# Methods for objects of class ridgeRes
################################################################################
#' @title Print ridgeRes object
#' @name ridgeRes
#' @export
#' @aliases summary.ridgeRes
#' @aliases print.ridgeRes
#' @aliases ridgeRes-class
#' @description Print a summary of the result of \code{\link{ridgeSIR}} (
#' \code{ridgeRes} object)
#' @param object a \code{ridgeRes} object
#' @param x a \code{ridgeRes} object
#' @param ... not used
#' @author {Victor Picheny, \email{victor.picheny@inrae.fr}\cr 
#' Remi Servien, \email{remi.servien@inrae.fr}\cr
#' Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}}
#' @seealso \code{\link{ridgeSIR}}
#' 
summary.ridgeRes <- function(object, ...) {
  cat("Ridge SIR results with:\n\n",
      object$parameters$H, "slices\n",
      "dimension of the EDR space is:", object$parameters$d, "\n",
      "regularization parameter is:", object$parameters$mu2, "\n")
  cat("The EDR space is in '$EDR'. First 10 rows are:\n\n")
  print(head(object$EDR, 10))
}

#' @export
#' @rdname ridgeRes
print.ridgeRes <- function(x, ...) {
  summary.ridgeRes(x)
}

################################################################################
# Cross validation for ridge SIR
################################################################################
#' @title Cross-Validation for ridge SIR
#'
#' @description
#' \code{tune.ridgeSIR} performs a Cross Validation for ridge SIR estimation
#'
#' @param x explanatory variables (numeric matrix or data frame)
#' @param y target variable (numeric vector)
#' @param listH list of the number of slices to be tested (numeric vector)
#' @param list_mu2 list of ridge regularization parameters to be tested 
#' (numeric vector)
#' @param list_d list of the dimensions to be tested (numeric vector)
#' @param nfolds number of folds for the cross validation. Default is 10
#' @param parallel whether the computation should be performed in parallel or
#' not. Logical. Default is FALSE
#' @param ncores number of cores to use if \code{parallel = TRUE}. If left to 
#' NULL, all available cores minus one are used
#' 
#' @author {Victor Picheny, \email{victor.picheny@inrae.fr}\cr
#' Remi Servien, \email{remi.servien@inrae.fr}\cr
#' Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}}
#' 
#' @references {Picheny, V., Servien, R. and Villa-Vialaneix, N. (2016) 
#' Interpretable sparse SIR for digitized functional data.
#' \emph{Statistics and Computing}, \strong{29}(2), 255--267.}
#' 
#' @seealso \code{\link{ridgeSIR}}
#' 
#' @examples
#' set.seed(1115)
#' tsteps <- seq(0, 1, length = 200)
#' nsim <- 100
#' simulate_bm <- function() return(c(0, cumsum(rnorm(length(tsteps)-1, sd=1))))
#' x <- t(replicate(nsim, simulate_bm()))
#' beta <- cbind(sin(tsteps*3*pi/2), sin(tsteps*5*pi/2))
#' y <- log(abs(x %*% beta[ ,1])) + sqrt(abs(x %*% beta[ ,2]))
#' y <- y + rnorm(nsim, sd = 0.1)
#' list_mu2 <- 10^(0:10)
#' listH <- c(5, 10)
#' list_d <- 1:4
#' set.seed(1129)
#' \donttest{
#' res_tune <- tune.ridgeSIR(x, y, listH, list_mu2, list_d, nfolds = 10, 
#'                           parallel = TRUE, ncores = 2)}
#' 
#' @return a data frame with tested parameters and corresponding CV error and 
#' estimation of R(d)
#' 
#' @export
#' 
tune.ridgeSIR <- function(x, y, listH, list_mu2, list_d, nfolds = 10, 
                          parallel = TRUE, ncores = NULL) {
  if (parallel) {
    if (is.null(ncores)) ncores <- min(c(detectCores() - 1, nfolds))
  }
  
  # create nfolds folds
  n <- nrow(x)
  size_fold <- round(n / nfolds)
  fold_num <- c(rep(1:(nfolds - 1), each = size_fold),
                rep(nfolds, n - (nfolds - 1) * size_fold))
  fold_num <- sample(fold_num, n, replace = FALSE)
  
  # initialization
  cv_errors <- NULL
  Rd <- NULL
  
  # loop over h
  for (h in listH) {
    print(paste("Processing h =", h))
    ## create slices
    slices <- makeSlices(y, h)
    
    ## loop over mu2 and d to obtain global projector (reference for Rd)
    max_d <- max(list_d)
    pre <- preSIR(x, slices, h)
    global_projector <- list()
    for (ind_mu2 in seq_along(list_mu2)) {
      mu2 <- list_mu2[ind_mu2]
      res_SIR <- processSIR(pre, max_d, mu2)
      global_projector[[ind_mu2]] <- sapply(list_d, function(d) {
        projectorEDR(res_SIR$EDR[ , 1:d, drop = FALSE], pre$ccov_x, mu2)
      }, simplify = FALSE)
    }
    
    ## compute this fold error
    ### parallel version
    if (parallel) {
      registerDoParallel(ncores)
      res <- foreach(cur_fold = 1:nfolds, .combine = "+",
                     .export = c("process_fold", "projectorEDR", "computeCV",
                                 "computeRd", "ridgeSIR", "preSIR",
                                 "processSIR", "makeSlices"),
                     .packages = c("expm", "RSpectra")) %dopar% {
        return(process_fold(cur_fold, x, slices$slices, fold_num, h, list_mu2,
                            list_d, global_projector))
                     }
      stopImplicitCluster()
    } else {### not parallel
      res <- foreach(cur_fold = 1:nfolds, .combine = "+") %do% {
        print(paste("process fold", cur_fold))
        return(process_fold(cur_fold, x, slices$slices, fold_num, h, list_mu2,
                            list_d, global_projector))
      }
    }
    cv_errors <- c(cv_errors, res[ ,1] / nfolds)
    Rd <- c(Rd, rep(list_d, length(list_mu2)) - res[ ,2] / nfolds)
  }
  
  lH <- length(listH)
  lmu2 <- length(list_mu2)
  ld <- length(list_d)
  out <- data.frame("H"   = rep(listH, each = lmu2 * ld), 
                    "mu2" = rep(rep(list_mu2, each = ld), lH),
                    "d"   = rep(list_d, lH * lmu2), 
                    "cverror" = cv_errors, "Rd" = Rd)
  
  return(out)
}

if(getRversion() >= "2.15.1")  utils::globalVariables(c("cur_fold"))

process_fold <- function(cur_fold, x, slices_y, fold_num, h, list_mu2, list_d,
                         global_projector) {
  # preprocessing common to all mu
  ## extract data
  test_ind <- which(fold_num == cur_fold)
  x_test <- x[test_ind, ]
  x_train <- x[- test_ind, ]
  slices_test <- slices_y[test_ind]
  freq_test <- table(slices_test) / length(slices_test)
  slices_train <- slices_y[- test_ind]
  freq_train <- table(slices_train) / length(slices_train)
  slices <- list("slices" = slices_train, "freq" = freq_train)
  ## SIR precomputations
  pre <- preSIR(x_train, slices, h)
  d_max <- max(list_d)
  
  # loop over mu2
  cv_errors <- NULL
  Rd <- NULL
  for (ind_mu2 in seq_along(list_mu2)) {
    # train on train data set
    mu2 <- list_mu2[ind_mu2]
    cur_model <- processSIR(pre = pre, d = d_max, mu2 = mu2)
    
    # compute CV error (based on MLR)
    unique_test <- sort(unique(slices_test))
    unique_train <- sort(unique(slices_train))
    mean_x <- colMeans(x_test)
    cmean_x <- lapply(split(as.data.frame(x_test), slices_test), colMeans)
    cmean_x <- matrix(unlist(cmean_x), ncol = length(unique_test))
    
    if (length(unique_test) < length(unique_train)) {
      # case where slices in test are not in train: the other case is not supported
      kept_train <- which(unique_train %in% unique_test)
    } else kept_train <- seq_along(unique_test)
    topred <- cmean_x - mean_x
    explanatory <- cov(x_test)
    smallest <- eigs(explanatory, 1, which = "SR")$value
    norm_test <- explanatory + abs(smallest) * 10^3 * diag(ncol(explanatory))
    norm_test <- forceSymmetric(norm_test)
    norm_test <- solve(norm_test)
    
    cv_errors <- c(cv_errors, sapply(list_d, function(d) {
      computeCV(topred, explanatory, freq_test, norm_test,
                cur_model$EDR[ ,1:d,drop = FALSE] %*% 
                  cur_model$condC[1:d,kept_train,drop = FALSE])
    }))
    
    # compute projection and Rd
    cur_projs <- sapply(list_d, function(d) {
      curEDR <- cur_model$EDR[ ,1:d,drop=FALSE]
      res <- projectorEDR(curEDR, pre$ccov_x, mu2)
    }, simplify = FALSE)
    Rd <- c(Rd, 
            unlist(mapply(computeRd, cur_projs, global_projector[[ind_mu2]])))
  }
  
  return(cbind(cv_errors, Rd))
}

projectorEDR <- function(EDR, ccov_x, mu2) {
  projector <- tcrossprod(EDR) %*% (ccov_x + mu2 * diag(nrow(EDR)))
  return(projector)
}

computeCV <- function(topred, explanatory, freq_test, norm, parameters) {
  quant <- topred - explanatory %*% parameters
  quant <- crossprod(quant, norm) %*% quant
  quant <- diag(quant) * freq_test
  
  return(sum(quant) / sum(freq_test))
}

computeRd <- function(projector, global) {
  sum(diag(projector %*% global))
}

################################################################################
# SFCB
################################################################################
#' @title sfcb
#' @name sfcb
#' @export
#'
#' @description
#' \code{sfcb} performs interval selection based on random forests
#'
#' @param X input predictors (matrix or data.frame)
#' @param Y target variable (vector whose length is equal to the number of rows
#' in X)
#' @param group.method group method. Default to \code{"adjclust"}
#' @param summary.method summary method. Default to \code{"pls"}
#' @param selection.method selection method. Default to \code{"none"} (no 
#' selection performed)
#' @param at number of groups targeted for output results (integer). Not used
#' when \code{range.at} is not \code{NULL}
#' @param range.at (vector of integer) sequence of the numbers of groups for
#' output results
#' @param seed random seed (integer)
#' @param repeats number of repeats for the final random forest computation
#' @param keep.time keep computational times for each step of the method? 
#' (logical; default to \code{TRUE})
#' @param verbose print messages? (logical; default to \code{TRUE})
#' @param parallel not implemented yet
#' 
#' @return an object of class \code{"SFCB"} with elements: 
#'   \item{dendro}{a dendrogram corresponding to the method chosen in
#'   \code{group.method}}
#'   \item{groups}{a list of length \code{length(range.at)} (or of length 1 if
#'   \code{range.at == NULL}) that contains the clusterings of input variables
#'   for the selected group numbers}
#'   \item{summaries}{a list of the same length than \code{$groups} that 
#'   contains the summarized predictors according to the method chosen in 
#'   \code{summary.methods}}
#'   \item{selected}{a list of the same length than \code{$groups} that contains
#'   the names of the variable selected by \code{selection.method} if it is not
#'   equal to \code{"none"}}
#'   \item{mse}{a data.frame with \code{repeats} \eqn{\times}
#'   \code{length($groups)} rows that contains Mean Squared Errors of the 
#'   \code{repeats} random forests fitted for each number of groups}
#'   \item{importance}{a list of the same length than \code{$groups} that 
#'   contains a data.frame providing variable importances for the variables in
#'   selected groups in \code{repeats} columns (one for each iteration of the
#'   random forest method). When \code{summary.method == "basics"}, importance 
#'   for mean and sd are provided in separated columns, in which case, the 
#'   number of columns is equal to 2\code{repeats}}
#'   \item{computational.times}{a vector with 4 values corresponding to the
#'   computational times of (respectively) the group, summary, selection, and RF
#'   steps. Only if \code{keep.time == TRUE}}
#'   \item{call}{function call}
#' 
#' @author Remi Servien, \email{remi.servien@inrae.fr}\cr
#' Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}
#' 
#' @references Servien, R. and Vialaneix, N. (2024) A random forest approach 
#' for interval selection in functional regression. \emph{Statistical Analysis
#' and Data Mining}, \strong{17}(4), e11705. \doi{10.1002/sam.11705}
#' 
#' @examples 
#' data(truffles)
#' out1 <- sfcb(rainfall, truffles, group.method = "adjclust", 
#'              summary.method = "pls", selection.method = "relief")
#' out2 <- sfcb(rainfall, truffles, group.method = "adjclust", 
#'              summary.method = "basics", selection.method = "none",
#'              range.at = c(5, 7))

sfcb <- function(X, Y, group.method = c("adjclust", "cclustofvar"), 
                 summary.method = c("pls", "basics", "cclustofvar"), 
                 selection.method = c("none", "boruta", "relief"), 
                 at = round(0.15 * ncol(X)), range.at = NULL,  seed = NULL, 
                 repeats = 5, keep.time = TRUE, verbose = TRUE, 
                 parallel = FALSE) {
  
  # Input checking ####
  group.method <- match.arg(group.method)
  summary.method <- match.arg(summary.method)
  selection.method <- match.arg(selection.method)
  if (is.null(range.at)) {
    ## checking 'at'
    at_length_one <- length(at) == 1
    at_numeric <- is.numeric(at)
    if (!at_length_one || !at_numeric) stop("'at' must be a positive integer")
    
    at_positive <- at > 0
    at_integer <- (at - floor(at)) == 0
    if (!at_positive || !at_integer) stop("'at' must be a positive integer")
  } else {
    ## checking 'range.at'
    range_length_two <- length(range.at) == 2
    range_numeric <- is.numeric(range.at)
    if (!range_length_two || !range_numeric) 
      stop(paste("'range.at' must be a vector of minimum and maximum numbers",
                 "of intervals (length 2) or 'NULL'"))
    
    range_positive <- range.at[1] > 0
    range_ordered <- range.at[1] < range.at[2]
    range_integer <- sapply(range.at, function(aat) (aat - floor(aat)) == 0)
    range_integer <- all(range_integer)
    if (!range_positive || !range_ordered || !range_integer) 
      stop(paste("'range.at' must be a vector of minimum and maximum numbers",
                 "of intervals (length 2) or 'NULL'"))
  }
  ## checking 'seed'
  if (!is.null(seed) && !is.numeric(seed)) stop("'seed' must be numeric!")
  
  ## checking Boruta availability
  if (selection.method == "boruta" && 
      !requireNamespace("Boruta", quietly = TRUE)) {
        stop("Package 'Boruta' needed for selection method 'boruta'.",
             call. = FALSE)
  }
  
  sfcb_call <- match.call()
  
  # Step1: group computation ####
  group_ct <- system.time({
    if (verbose) message("Creating groups...")
    group_function <- sprintf("group_%s", group.method)
    group_function <- eval(as.name(group_function))
    all_groups <- group_function(X)
  })[3]
  
  # Step2: compute summaries ####
  summary_ct <- system.time({
    if (verbose) message("Computing summaries...")
    if (is.null(range.at)) {
      selected_groups <- all_groups$groups[as.character(at)]
    } else {
      selected <- as.character(range.at[1]:range.at[2])
      selected_groups <- all_groups$groups[selected]
    }
    summary_function <- sprintf("summary_%s", summary.method)
    summary_function <- eval(as.name(summary_function))
    function_inputs <- names(formals(summary_function))
    if ("target" %in% function_inputs) {
      suppressMessages({
        all_summaries <- summary_function(X, Y, selected_groups)
      })
    } else {
      suppressMessages({all_summaries <- summary_function(X, selected_groups)})
    }
    if (summary.method != "basics") {
      all_summaries <- lapply(all_summaries, function(as) {
        out <- as.data.frame(as)
        names(out) <- paste0("g", 1:ncol(as))
        return(out)
      })
    } else {
      all_summaries <- lapply(all_summaries, function(as) {
        out <- as.data.frame(as)
        names(out) <- gsub("clustmean", "mean\\.g", names(out))
        names(out) <- gsub("clustsd", "sd\\.g", names(out))
        return(out)
      })
    }
  })[3]
  
  # Step 3: selection ####
  if (selection.method != "none") {
    selection_ct <- system.time({
      if (verbose) message("Computing selections...")
      
      selection_function <- sprintf("selection_%s", selection.method)
      selection_function <- eval(as.name(selection_function))
      if ("seed" %in% names(formals(selection_function))) {
        all_selections <- selection_function(all_summaries, Y, seed)
      } else all_selections <- selection_function(all_summaries, Y)
      
      # turn selections into their original variables (not at summary level)
      if (summary.method != "basics") {
        orig_selected <- mapply(summary2original, all_selections, 
                                selected_groups, 
                                MoreArgs = list("varnames" = names(X)),
                                SIMPLIFY = FALSE)
      } else {
        orig_selected <- mapply(summary2originalb, all_selections, 
                                selected_groups, all_summaries, 
                                MoreArgs = list("varnames" = names(X)),
                                SIMPLIFY = FALSE)
      }
    })[3]
  } else {
    all_selections <- "none"
    selection_ct <- 0
  }
  
  # Step 4: prediction ####
  rf_ct <- system.time({
    if (verbose) message("Fitting random forest...")
    all_qualities <- predict_rf(all_summaries, Y, selected_groups,
                                all_selections, repeats, seed,
                                (summary.method == "basics"), names(X))
  })[3]
  
  # Output preparation
  if (selection.method != "none") {
    out <- list("dendro" = all_groups$dendro, "groups" = selected_groups,
                "summaries" = all_summaries, "selected" = orig_selected,
                "mse" = all_qualities$mse, 
                "importances" = all_qualities$importances)
  } else {
    out <- list("dendro" = all_groups$dendro, "groups" = selected_groups,
                "summaries" = all_summaries, "mse" = all_qualities$mse, 
                "importances" = all_qualities$importances)
  }
  if (keep.time) {
    out$"computational.times" <- c(group_ct, summary_ct, selection_ct, rf_ct)
    names(out$"computational.times") <- c("group", "summary", "selection", "RF")
  }
  out$call <- sfcb_call
  
  class(out) <- "SFCB"
  
  return(out)
}

library("SISIR")

context("Test that extract works as expected...")

data("truffles")

test_that("extract works as expected without selection.", {
  expected_outputs <- c("dendro", "groups", "summaries", "mse", "importances",
                        "call")
  
  out1 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", range.at = c(5, 7))
  out1b <- extract_at(out1, 5)
  expect_named(out1b, expected_outputs)
  expect_length(out1b$groups, 1)
  expect_length(out1b$summaries, 1)
  expect_length(out1b$importances, 1)
  expect_equal(nrow(out1b$mse), 5)
  
  out2 <- sfcb(rainfall, truffles, group.method = "cclustofvar", 
               summary.method = "cclustofvar", range.at = c(5, 7))
  out2b <- extract_at(out2, 5)
  expect_named(out2b, expected_outputs)
  
  out3 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "basics", range.at = c(5, 7))
  out3b <- extract_at(out3, 5:6)
  expect_named(out3b, expected_outputs)
  expect_length(out3b$groups, 2)
  expect_length(out3b$summaries, 2)
  expect_length(out3b$importances, 2)
  expect_equal(nrow(out3b$mse), 5 * 2)
})

test_that("extract works as expected with selection.", {
  expected_outputs <- c("dendro", "groups", "summaries", "selected", "mse", 
                        "importances", "call")
  
  out4 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", selection.method = "relief", 
               range.at = c(5, 7))
  out4b <- extract_at(out4, 6)
  expect_named(out4b, expected_outputs)
  
  out5 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", selection.method = "relief", 
               range.at = c(5, 12))
  out5b <- extract_at(out5, c(9, 11:12))
  expect_named(out5b, expected_outputs)
  expect_length(out5b$groups, 3)
  expect_length(out5b$summaries, 3)
  expect_length(out5b$importances, 3)
  expect_equal(nrow(out5b$mse), 5 * 3)
})

test_that("extract returns errors as expected.", {
  out1 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", range.at = c(5, 7))
  expect_error({ extract_at(out1, "A") }, "'at' must be a numeric vector",
               fixed = FALSE)
  expect_error({ extract_at(out1, 9) }, 
               "'at' must be included in the range of tested groups for",
               fixed = FALSE)
})

library("SISIR")

context("Test that `sfcb` works as expected...")

data("truffles")

test_that("`sfcb` works for one `at` and no selection.", {
  expected_outputs <- c("dendro", "groups", "summaries", "mse", "importances",
                        "computational.times", "call")
  
  out1 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", seed = 3)
  expect_named(out1, expected_outputs)
  out1b <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", seed = 3)
  out1$"computational.times" <- NULL
  out1b$"computational.times" <- NULL
  expect_identical(out1, out1b)
  
  out2 <- sfcb(rainfall, truffles, group.method = "cclustofvar", 
               summary.method = "cclustofvar", keep.time = FALSE)
  expect_named(out2, setdiff(expected_outputs, "computational.times"))
  
  out3 <- sfcb(rainfall, truffles, group.method = "cclustofvar", 
               summary.method = "basics", repeats = 6)
  expect_named(out3, expected_outputs)
  expect_equal(nrow(out3$mse), 6)
  expect_equal(ncol(out3$importances[[1]]), 6 * 2)
})

test_that("`sfcb` works for `range_at` and no selection.", {
  expected_outputs <- c("dendro", "groups", "summaries", "mse", "importances",
                        "computational.times", "call")
  
  out4 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", range.at = c(5, 7))
  expect_named(out4, expected_outputs)
  expect_length(out4$summaries, 3)
  
  out5 <- sfcb(rainfall, truffles, group.method = "cclustofvar", 
               summary.method = "cclustofvar", range.at = c(5, 7))
  expect_named(out5, expected_outputs)
  
  out6 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "basics", range.at = c(5, 7))
  expect_named(out6, expected_outputs)
})

test_that("`sfcb` works for one `at` with selection.", {
  expected_outputs <- c("dendro", "groups", "summaries", "selected", "mse", 
                        "importances", "computational.times", "call")
  
  out7 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", selection.method = "relief")
  expect_named(out7, expected_outputs)
  
  out8 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", selection.method = "relief")
  expect_named(out8, expected_outputs)
  
  out9 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "basics", selection.method = "relief")
  expect_named(out9, expected_outputs)
  
  if (requireNamespace("Boruta", quietly = TRUE)) {
    out10 <- sfcb(rainfall, truffles, group.method = "adjclust", 
                  summary.method = "pls", selection.method = "boruta")
    expect_named(out10, expected_outputs)
  }
})

test_that("`sfcb` works for one `at` with selection.", {
  expected_outputs <- c("dendro", "groups", "summaries", "selected", "mse", 
                        "importances", "computational.times", "call")
  
  out11 <- sfcb(rainfall, truffles, group.method = "adjclust", 
                summary.method = "pls", selection.method = "relief", 
                range.at = c(5, 7))
  expect_named(out11, expected_outputs)
  expect_length(out11$selected, 3)
  
  if (requireNamespace("Boruta", quietly = TRUE)) {
    out12 <- sfcb(rainfall, truffles, group.method = "adjclust", 
                  summary.method = "pls", selection.method = "boruta", seed = 3)
    expect_named(out12, expected_outputs)
  }
  
  out13 <- sfcb(rainfall, truffles, group.method = "adjclust", 
                summary.method = "basics", selection.method = "relief", 
                range.at = c(5, 7))
  expect_named(out13, expected_outputs)
  
  out14 <- sfcb(rainfall, truffles, group.method = "adjclust", 
                summary.method = "pls", selection.method = "relief", 
                range.at = c(5, 12))
  expect_named(out14, expected_outputs)
})

test_that("`sfcb` properly returns error when expected.", {
  expect_error({sfcb(rainfall, truffles, at = "1")},
               "'at' must be a positive integer", fixed = FALSE)
  expect_error({sfcb(rainfall, truffles, at = -3)},
               "'at' must be a positive integer", fixed = FALSE)
  expect_error({sfcb(rainfall, truffles, at = 2.4)},
               "'at' must be a positive integer", fixed = FALSE)
  
  expect_error({sfcb(rainfall, truffles, range.at = 1:3)},
               "'range.at' must be a vector of minimum and maximum numbers", 
               fixed = FALSE)
  expect_error({sfcb(rainfall, truffles, range.at = c("1", "2"))},
               "'range.at' must be a vector of minimum and maximum numbers", 
               fixed = FALSE)
  expect_error({sfcb(rainfall, truffles, range.at = c(-1, 2))},
               "'range.at' must be a vector of minimum and maximum numbers", 
               fixed = FALSE)
  expect_error({sfcb(rainfall, truffles, range.at = c(1.4, 2))},
               "'range.at' must be a vector of minimum and maximum numbers", 
               fixed = FALSE)
  expect_error({sfcb(rainfall, truffles, range.at = c(5, 2))},
               "'range.at' must be a vector of minimum and maximum numbers", 
               fixed = FALSE)
  
  expect_error({sfcb(rainfall, truffles, seed = "1")}, 
               "'seed' must be numeric!", fixed = FALSE)
})

library("SISIR")

context("Test that graphics for `SFCB` objects work as expected...")

data("truffles")

test_that("graphics for 'importance' work as expected.", {
  
  out1 <- sfcb(rainfall, truffles, group.method = "adjclust",
               summary.method = "pls")
  p <- plot(out1, plot.type = "importance")
  expect_s3_class(p, "ggplot")
  p <- plot(out1, plot.type = "importance", shape.imp = "histogram")
  expect_s3_class(p, "ggplot")
  
  out2 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", selection.method = "relief", 
               range.at = c(5, 12))
  p <- plot(out2, plot.type = "importance")
  expect_s3_class(p, "ggplot")
  p <- plot(out2, plot.type = "importance", shape.imp = "histogram")
  expect_s3_class(p, "ggplot")
  
  out3 <- sfcb(rainfall, truffles, group.method = "cclustofvar", 
               summary.method = "basics")
  p <- plot(out3, plot.type = "importance")
  expect_s3_class(p, "ggplot")
  p <- plot(out3, plot.type = "importance", shape.imp = "histogram")
  expect_s3_class(p, "ggplot")
  
  out4 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "basics", range.at = c(5, 7))
  p <- plot(out4, plot.type = "importance")
  expect_s3_class(p, "ggplot")
  p <- plot(out4, plot.type = "importance", shape.imp = "histogram")
  expect_s3_class(p, "ggplot")
  
  out5 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "basics", selection.method = "relief", 
               range.at = c(5, 7))
  p <- plot(out5, plot.type = "importance")
  expect_s3_class(p, "ggplot")
  p <- plot(out5, plot.type = "importance", shape.imp = "histogram")
  expect_s3_class(p, "ggplot")
  
})

test_that("graphics for 'selection' work as expected.", {
  
  out1 <- sfcb(rainfall, truffles, group.method = "adjclust",
               summary.method = "pls")
  p <- plot(out1, plot.type = "selection")
  expect_s3_class(p, "ggplot")
  p <- plot(out1, plot.type = "selection", sel.type = "selection", 
            threshold = 0.00011)
  expect_s3_class(p, "ggplot")
  
  out2 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", selection.method = "relief")
  p <- plot(out2, plot.type = "selection")
  expect_s3_class(p, "ggplot")
  p <- plot(out2, plot.type = "selection", sel.type = "selection")
  expect_s3_class(p, "ggplot")
  
  out3 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", selection.method = "relief", 
               range.at = c(5, 12))
  p <- plot(out3, plot.type = "selection")
  expect_s3_class(p, "ggplot")
  p <- plot(out3, plot.type = "selection", sel.type = "selection")
  expect_s3_class(p, "ggplot")
  
  out4 <- sfcb(rainfall, truffles, group.method = "cclustofvar", 
               summary.method = "basics")
  p <- plot(out4, plot.type = "selection")
  expect_s3_class(p, "ggplot")
  p <- plot(out4, plot.type = "selection", sel.type = "selection", 
            threshold = 0.0015)
  expect_s3_class(p, "ggplot")
  
  out5 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "basics", range.at = c(5, 7))
  suppressWarnings({ p <- plot(out5, plot.type = "selection") })
  expect_s3_class(p, "ggplot")
  p <- plot(out5, plot.type = "selection", sel.type = "selection", 
            threshold = 0.005)
  expect_s3_class(p, "ggplot")
  
  out6 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "basics", selection.method = "relief")
  p <- plot(out6, plot.type = "selection", sel.type = "selection")
  expect_s3_class(p, "ggplot")
  
  out7 <- sfcb(rainfall, truffles, group.method = "adjclust", 
                summary.method = "basics", selection.method = "relief", 
                range.at = c(5, 7))
  p <- plot(out7, plot.type = "selection", sel.type = "selection")
  expect_s3_class(p, "ggplot")
  
  out8 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", range.at = c(5, 7))
  p <- plot(out8, plot.type = "selection", sel.type = "selection", 
            threshold = 0.02)
  expect_s3_class(p, "ggplot")
  
  expect_error({ p <- plot(out8, plot.type = "selection", 
                           sel.type = "selection", threshold = "1") },
               "'threshold' must be numeric.", fixed = FALSE)
  expect_error({ p <- plot(out7, plot.type = "selection", 
                           sel.type = "selection", threshold = 1) },
               "A selection method has already been used", fixed = FALSE)
  expect_warning({ p <- plot(out4, plot.type = "selection",
                             threshold = 0.0015) },
                 "'sel.type' is not 'selection'. Automatically switching", 
                 fixed = FALSE)
  expect_error({ p <- plot(out4, plot.type = "selection", 
                           sel.type = "selection") },
               "Choose sel.type = 'importance'", fixed = FALSE)
  
})

test_that("graphics for 'dendrogram' work as expected.", {
  out1 <- sfcb(rainfall, truffles, group.method = "adjclust",
               summary.method = "pls")
  expect_message({ plot(out1) }, "Reversals detected in the dendrogram.")
  
  out2 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", selection.method = "relief")
  expect_message({ plot(out2) }, "Reversals detected in the dendrogram.")
  
  out3 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", selection.method = "relief", 
               range.at = c(5, 12))
  expect_message({ plot(out3) }, "Only the first 3 selections are represented")
  
  out4 <- sfcb(rainfall, truffles, group.method = "cclustofvar", 
               summary.method = "basics")
  expect_message({ plot(out4) }, "Reversals detected in the dendrogram.")
})

#' @importFrom CORElearn attrEval
#' @importFrom utils packageVersion

selection_boruta <- function(summaries, Y, seed) {
  # Description #####
  # returns a list with selected indices for each level of the hierarchy of
  # groups and summaries (Boruta method)
  # Inputs #####
  # summaries: a list with summaries obtained for each level of the hierarchy of
  # groups
  # Y: corresponding target
  # Outputs #####
  # a list with selected indices (same length as summaries)
  
  selections <- sapply(summaries, simplify = FALSE, function(curX) {
    all_indices <- 1:ncol(curX)
    has_nona <- colSums(is.na(curX)) == 0
    curX <- curX[, has_nona, drop = FALSE]
    cur_df <- data.frame(curX, "Y" = Y)
    if (!is.null(seed)) set.seed(seed)
    if (packageVersion("Boruta") >= "10.0.0") {
      boruta_decision <- Boruta::Boruta(Y ~ ., data = cur_df, threads = 1)
    } else {
      boruta_decision <- Boruta::Boruta(Y ~ ., data = cur_df, num.threads = 1)
    }
    boruta_decision <- boruta_decision$finalDecision
    out <- which(boruta_decision %in% c("Tentative", "Confirmed"))
    out <- all_indices[has_nona][out]
    out <- unname(out)
    
    return(out)
  })
  
  return(selections)
}

selection_relief <- function(summaries, Y) {
  # Description #####
  # returns a list with selected indices for each level of the hierarchy of
  # groups and summaries (Relief method)
  # Inputs #####
  # summaries: a list with summaries obtained for each level of the hierarchy of
  # groups
  # Y: corresponding target
  # Outputs #####
  # a list with selected indices (same length as summaries)
  
  selections <- sapply(summaries, simplify = FALSE, function(curX) {
    all_indices <- 1:ncol(curX)
    has_nona <- colSums(is.na(curX)) == 0
    curX <- curX[, has_nona, drop = FALSE]
    df <- data.frame("y" = Y, curX)
    eval_f <- CORElearn::attrEval(y ~ ., data = df, 
                                  estimator = "RReliefFexpRank")
    evol <- eval_f[order(eval_f, decreasing = TRUE)] + min(eval_f)
    n <- ncol(df) - 1
    nb_pos <- max(c(1, sum(eval_f > 0)))
    tot_evol <- sum(evol)
    bs <- tot_evol * rev(cumsum(1/(n:1))/n)
    cdt <- which(evol <= bs)
    if (length(cdt) > 0) {
      nb <- min(c(min(cdt), nb_pos))
    } else nb <- nb_pos
    ordered_f <- length(eval_f) + 1 - rank(eval_f)
    out <- which(ordered_f <= nb)
    out <- all_indices[has_nona][out]
    out <- unname(out)
    
    return(out)
  })
  
  return(selections)
}

#' @importFrom stats cor
#' @importFrom adjclust adjClust cutree_chac

group_adjclust <- function(dataset) {
  # Description #####
  # returns a hierarchy of groups (list of clusterings) obtained by cutting the 
  # dendrogram provided by adjclust on 'dataset'
  # Inputs #####
  # dataset: a data.frame with observations in rows and variables in columns
  # Outputs #####
  # a list where each entry is a clustering named by the number of clusters
  # ordered from the largest number of clusters (number of variables) to 1
  
  corr_inputs <- stats::cor(dataset)
  hclust_res <- adjclust::adjClust(corr_inputs, type = "similarity")
  groups <- lapply(1:ncol(dataset), function(k) 
    adjclust::cutree_chac(hclust_res, k = k))
  hclust_res <- as.hclust(hclust_res)
  names(groups) <- 1:ncol(dataset)
  groups <- groups[ncol(dataset):1]
  
  return(list("groups" = groups, "dendro" = hclust_res))
}

group_cclustofvar <- function(dataset) {
  # Description #####
  # returns a hierarchy of groups (list of clusterings) obtained by cutting the 
  # dendrogram provided by an order constrained version of clustofvar on 
  # 'dataset'
  # Inputs #####
  # dataset: a data.frame with observations in rows and variables in columns
  # Outputs #####
  # a list where each entry is a clustering named by the number of clusters
  # ordered from the largest number of clusters (number of variables) to 1
  
  # initialization
  p <- ncol(dataset)
  inputs <- scale(dataset) / sqrt(nrow(dataset)-1)
  clust_res <- data.frame("number" = rep(0, p-1),
                          "left" = rep(0, p-1),
                          "right" = rep(0, p-1),
                          "link" = rep(0, p-1))
  
  all_groups <- as.list(1:p)
  names(all_groups) <- as.character(1:p)
  old_group_homogeneity <- rep(1, length(all_groups)) # because initial homogeneity is 1
  groups_homogeneity <- sapply(1:(length(all_groups)-1), function(ind) {
    selected <- c(all_groups[[ind]], all_groups[[ind+1]])
    homogeneity(dataset[, selected])
  })
  
  # loop over the number of clusters for merges
  for (level in 1:(p-1)) {
    ng <- length(old_group_homogeneity)
    criteria <- old_group_homogeneity[1:(ng-1)] + old_group_homogeneity[2:ng] - 
      groups_homogeneity 
    to_group <- which.min(criteria)
    cur_nei <- as.numeric(names(all_groups)[c(to_group, to_group+1)])
    clust_res[level, ] <- c(-level, cur_nei, min(criteria))
    # update left and right homogeneity
    old_group_homogeneity <- groups_homogeneity
    if (level < p-1) {
      if (to_group > 1) {
        selected <- c(all_groups[[(to_group-1)]], all_groups[[to_group]],
                      all_groups[[to_group+1]])
        left_homo <- homogeneity(dataset[, selected])
        groups_homogeneity[to_group - 1] <- left_homo
        if (to_group < length(all_groups) - 1) {
          selected <- c(all_groups[[(to_group+2)]], all_groups[[to_group]],
                        all_groups[[to_group+1]])
          right_homo <- homogeneity(dataset[, selected])
          groups_homogeneity[to_group] <- right_homo
          groups_homogeneity <- groups_homogeneity[-c(to_group+1)]
        } else {
          groups_homogeneity <- groups_homogeneity[-to_group]
        }
      } else {
        selected <- c(all_groups[[(to_group+2)]], all_groups[[to_group]],
                      all_groups[[to_group+1]])
        right_homo <- homogeneity(dataset[, selected])
        groups_homogeneity[to_group+1] <- right_homo
        groups_homogeneity <- groups_homogeneity[-1]
      }
    }
    
    # update groups
    all_groups[[to_group]] <- c(all_groups[[to_group]], 
                                all_groups[[to_group+1]])
    names(all_groups)[to_group] <- -level
    all_groups <- all_groups[-c(to_group+1)]
  }
  
  clust_res[, 2:3] <- - clust_res[, 2:3]
  out_clust <- list("merge" = matrix(c(as.integer(clust_res[, 2]),
                                       as.integer(clust_res[, 3])),
                                     ncol = 2, byrow = FALSE),
                    "height" = clust_res$link,
                    "order" = 1:p,
                    "labels" = names(dataset),
                    "method" = "CClustOfVar",
                    "call" = NULL,
                    "dist.method" = "PCAmix")
  class(out_clust) <- "chac"
  
  groups <- lapply(1:ncol(dataset), function(k) 
    adjclust::cutree_chac(out_clust, k = k))
  names(groups) <- 1:ncol(dataset)
  groups <- groups[ncol(dataset):1]
  
  out_clust <- as.hclust(out_clust)
  
  return(list("groups" = groups, "dendro" = out_clust))
}

# useful functions #####
homogeneity <- function(mat) { 
  # compute homogeneity criterion (first svd value)
  out <- svd(mat, 1, 1)
  return(out$d[1])
}

################################################################################
# Documentation for the dataset "truffles"
################################################################################

#' @title Dataset "Truffles"
#' @name truffles
#' @aliases rainfall beta
#' 
#' @description Yearly truffles production and corresponding monthly rainfall 
#' information of the Perigord black truffle in the Vaucluse (France) between
#' 1924 and 1949.
#' 
#' @docType data
#' 
#' @format 3 datasets are provided: \itemize{
#'   \item \code{rainfall}: a data frame with 15 columns (months from January 
#'   Year n to March Year n+1) and 25 rows (production years from 1924/1925 to 
#'   1948/1949). Data correspond to cumulated rainfall in mm;
#'   \item \code{truffles}: a vector with 25 values corresponding to the total 
#'   production (in kg) of truffles in the truffle patch of T. melanosporum de 
#'   Pernes-Les-Fontaines (Vaucluse, France);
#'   \item \code{beta}: 0/1 vector with 15 values indicated the months during 
#'   which the rainfall has the most important influence on the truffle 
#'   production, as provided by experts.
#' }
#' 
#' @details This dataset has been made available by courtesy of the authors of 
#' the publication [Baragatti \emph{et al.}, 2019]. Meteorological data have 
#' been provided by Meteo France https://meteofrance.com (Orange meteorological 
#' station) and truffle production data are courtesy of the truffle patch.
#' 
#' @references
#' Baragatti M., Grollemund P.M., Montpied P., Dupouey J.L., Gravier J., Murat 
#' C., Le Tacon F. (2019) Influence of annual climatic variations, climate
#' changes, and sociological factors on the production of the Perigord black 
#' truffle (\emph{Tuber melanosporum Vittad.}) from 1903-1904 to 1988-1989 in 
#' the Vaucluse (France), \emph{Mycorrhiza}, \strong{29}(2), 113-125.
#' 
#' @examples 
#' data(truffles)
#' summary(truffles)
#' plot(1:15, rainfall[1, ], type = "l", xlab = "month", ylab = "rainfall (mm)")

NULL

# Methods for SFCB-class ####
#' @title Methods for SFCB objects
#' @name SFCB-class
#' @export
#' @aliases summary.SFCB
#' @aliases print.SFCB
#' @aliases plot.SFCB
#' @aliases extract_at.SFCB
#' @aliases quality.SFCB
#' @aliases extract_at
#' @aliases quality
#' @aliases SFCB-class
#' @description Print, plot, manipulate or compute quality for outputs of the
#' \code{\link{sfcb}} function (\code{SFCB} object)
#' @param object a \code{SFCB} object
#' @param x a \code{SFCB} object
#' @param ... not used
#' @param plot.type type of the plot. Default to \code{"dendrogram"} (see
#' Details)
#' @param sel.type when \code{plot.type == "selection"}, criterion on which to
#' base the selection. Default to \code{"importance"}
#' @param threshold numeric. When \code{plot.type == "importance"}, numeric
#' threshold to perform a selection (if none has been done before for \code{x})
#' based on thresholding of importance. Default to \code{"none"} in which case
#' no thresholding is performed
#' @param shape.imp when \code{plot.type == "importance"}, type of plot to
#' represent the importance. Default to \code{"boxplot"}
#' @param quality.crit character vector (length 1 or 2) indicating one or two
#' quality criteria to display. The values have to be taken in \{\code{"mse"},
#' \code{"time"}, \code{"Precision"}, \code{"Recall"}, \code{"ARI"}, 
#' \code{"NMI"}\}. If \code{"time"} is chosen, it can not be associated with any
#' other criterion
#' @param ... not used
#' @param at numeric vector. Set of the number of intervals to extract for 
#' @param ground_truth numeric vector of ground truth. Target variables 
#' to compute qualities correspond to non-zero entries of this vector
#' @param threshold numeric value. If not \code{NULL}, selection of variables to
#' compute qualities is based on a threshold of importance values
#' \code{extract_at}
#' @author {Remi Servien, \email{remi.servien@inrae.fr}\cr
#' Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}}
#' @references {Servien, R. and Vialaneix, N. (2023) A random forest approach 
#' for interval selection in functional regression. Preprint.}
#' @details The \code{plot} functions can be used in four different ways to 
#' extract information from the \code{SFCB} object: \itemize{
#'  \item \code{plot.type == "dendrogram"} displays the dendrogram obtained at
#'  the clustering step of the method. Depending on the cases, the dendrogram
#'  comes with additional information on clusters, variable selections and/or
#'  importance values;
#'  \item \code{plot.type == "selection"} displays either the evolution of the
#'  importance for the simulation with the best (smallest) MSE for each time
#'  step in the range of the functional predictor or the evolution of the
#'  selected intervals along the whole range of the functional prediction also 
#'  for the best MSE;
#'  \item \code{plot.type == "importance"} displays a summary of the importance
#'  values over the whole range of the functional predictor and for the 
#'  different experiments. This summary can take the form of a boxplot or of
#'  an histogram;
#'  \item \code{plot.type == "quality"} displays one or two quality distribution
#'  with respect to the different experiments and different number of intervals.
#' }
#' @seealso \code{\link{sfcb}}
#' @examples 
#' data(truffles)
#' out1 <- sfcb(rainfall, truffles, group.method = "adjclust", 
#'              summary.method = "pls", selection.method = "relief")
#' summary(out1)
#' 
#' plot(out1)
#' plot(out1, plot.type = "selection")
#' plot(out1, plot.type = "importance")
#' 
#' out2 <- sfcb(rainfall, truffles, group.method = "adjclust", 
#'              summary.method = "basics", selection.method = "none",
#'              range.at = c(5, 7))
#' out3 <- extract_at(out2, at = 6)
#' summary(out3)
#' 
summary.SFCB <- function(object, ...) {
  cat("\nCall:\n")
  print(object$call)
  cat("\nSFCB object with:\n")
  lobj <- length(object$groups)
  if (lobj == 1) {
    cat("    -", length(unique(object$groups[[1]])), "interval(s)\n")
  } else {
    minl <- length(unique(object$groups[[1]]))
    maxl <- length(unique(object$groups[[lobj]]))
    cat("    -", minl, "-", maxl, "interval(s)\n")
  }
  if ("selected" %in% names(object)) {
    if (!is.null(ncol(object$selected[[1]]))) {
      selected <- lapply(object$selected, function(alist) {
        apply(alist[, 2:3], 1, sum)
      })
    } else selected <- object$selected
    if (lobj == 1) {
      selected <- object$groups[[1]][selected[[1]]]
      cat("    -", length(unique(selected)), "selected interval(s)\n")
    } else {
      selected <- mapply(function(a, b) a[b], object$groups, selected)
      selected <- sapply(selected, function(x) length(unique(x)))
      cat("    -", min(selected), "-", max(selected), "selected interval(s)\n")
    }
  }
  cat("    -", nrow(object$mse) / lobj, "repeats\n")
  cat("    - MSE ranging in [", min(object$mse$mse), ", ", max(object$mse$mse),
      "]\n", sep = "")
  if ("computational.times" %in% names(object)) {
    cat("    - computational time (total):", sum(object$computational.times), 
        "(seconds)\n\n")
  }
  if ("quality" %in% names(object)) {
    cat("    - precision wrt ground truth in [", 
        min(object$quality$Precision, na.rm = TRUE), ", ", 
        max(object$quality$Precision, na.rm = TRUE), "]\n", sep = "")
    cat("    - recall wrt ground truth in [", 
        min(object$quality$Recall, na.rm = TRUE), ", ", 
        max(object$quality$Recall, na.rm = TRUE), "]\n", sep = "")
    if ("threshold" %in% names(object))
      cat("    for threshold: ", object$threshold, "\n", sep = "")
    cat("\n")
  }
  return(invisible(NULL))
}

#' @export
#' @rdname SFCB-class
print.SFCB <- function(x, ...) {
  summary(x)
}

#' @rdname SFCB-class
#' @export
plot.SFCB <- function(x, ...,
                      plot.type = c("dendrogram", "selection", "importance",
                                    "quality"),
                      sel.type = c("importance", "selection"),
                      threshold = "none", shape.imp = c("boxplot", "histogram"),
                      quality.crit = "mse") {
  args <- list("x" = x)
  plot.type <- match.arg(plot.type)
  args$"shape.imp" <- match.arg(shape.imp)
  args$"sel.type" <- match.arg(sel.type)
  args$"threshold" <- threshold
  args$"quality.crit" <- quality.crit
  
  plot_function <- sprintf("plot_%s", plot.type)
  plot_function <- eval(as.name(plot_function))
  args <- args[names(formals(plot_function))]
  
  p <- do.call("plot_function", args)
  if (plot.type != "dendrogram") return(p)
  
  return(invisible())
}

#' @rdname SFCB-class
#' @export
extract_at <- function(object, at) {
  UseMethod("extract_at")
}

#' @export
extract_at.SFCB <- function(object, at) {
  
  extract_call <- match.call()
  
  if (!is.numeric(at)) stop("'at' must be a numeric vector")
  orig_at <- names(object$groups)
  at <- sort(at)
  selected <- match(at, orig_at)
  if (anyNA(selected)) {
    stop("'at' must be included in the range of tested groups for 'object'.")
  }
  
  out <- object
  out$groups <- out$groups[selected]
  out$summaries <- out$summaries[selected]
  out$mse <- out$mse[out$mse$clust %in% at, ]
  out$importances <- out$importances[selected]
  if ("computational.times" %in% names(out)) out$"computational.times" <- NULL
  if ("selected" %in% names(out)) out$selected <- out$selected[selected]
  
  out$call <- extract_call
  
  return(out)
}

#' @rdname SFCB-class
#' @export
quality <- function(object, ground_truth, threshold = NULL) {
  UseMethod("quality")
}

#' @export
#' @importFrom aricode ARI NID NMI AMI NVI
quality.SFCB <- function(object, ground_truth, threshold = NULL) {
  
  if (length(ground_truth) != length(object$groups[[1]]))
    stop(paste("'ground_truth' must have a length identical to initial number ",
               "of variables."))

  if (!is.null(threshold) && (!is.numeric(threshold) || threshold <= 0)) 
    stop("'threshold' must be a positive number or NULL.")
  
  if (is.null(threshold) & !("selected" %in% names(object))) 
    stop(paste("No selected interval in this 'SFCB' object and no 'threshold'",
               "provided. Can not compute quality measures..."))
  
  all_at <- unique(object$mse$clust)
  basics <- ncol(object$importances[[1]]) != (nrow(object$mse) / length(all_at))
  out_obj <- object
  
  ground_truth_f <- sapply(ground_truth != 0, ifelse, yes = 1, no = 0)
  ground_truth_f <- factor(ground_truth_f)
  range_names <- names(object$groups)
  var_names <- names(object$groups[[1]])
  if (!is.null(threshold)) {
    if (basics) {
      object$importances <- lapply(object$importances, function(alist) {
        colmean <- grep("mean", colnames(alist))
        colsd <- grep("sd", colnames(alist))
        impm <- lapply(seq_along(colmean), function(ind) {
          pmax(alist[, colmean[ind]], alist[, colsd[ind]], na.rm = TRUE)
        })
        impm <- Reduce(cbind, impm)
        rownames(impm) <- rownames(alist)
        return(impm)
      })
    }
    nbrep <- ncol(object$importances[[1]])
    out <- lapply(object$importances, function(imp) {
      quals <-  lapply(1:ncol(imp), 
                       function(col) rownames(imp)[imp[, col] > threshold])
      quals <- lapply(quals, compute_qualities, var_names, ground_truth_f)
      quals <- Reduce(rbind, quals)
      return(quals)
    })
    out <- data.frame("clust" = rep(range_names, sapply(out, nrow)),
                      "repeats" = rep(1:nbrep, length(all_at)),
                      Reduce(rbind, out))
    rownames(out) <- NULL
  } else {
    if (basics) {
      object$selected <- lapply(object$selected, function(alist) {
        out <- alist$"in.mean" | alist$"in.sd"
        out <- alist$variable[out]
        return(out)
      })
    }
    out <- lapply(object$selected, compute_qualities, var_names = var_names, 
                  ground_truth_f = ground_truth_f)
    out <- data.frame("clust" = range_names, Reduce(rbind, out))
    rownames(out) <- NULL
  }
  
  out_obj$truth <- ground_truth
  out_obj$quality <- out
  if (!is.null(threshold)) out_obj$threshold <- threshold
  return(out_obj)
}

compute_qualities <- function(asel, var_names, ground_truth_f) {
  sel_f <- sapply(var_names %in% asel, ifelse, yes = 1, no = 0)
  sel_f <- factor(sel_f)
  nbpos <- sum(sel_f == "1" & ground_truth_f == "1")
  tpr <- nbpos / sum(sel_f == "1")
  recall <- nbpos / sum(ground_truth_f == "1")
  ari <- ARI(sel_f, ground_truth_f)
  nmi <- NMI(sel_f, ground_truth_f)
  out <- data.frame("Precision" = tpr, "Recall" = recall, "ARI" = ari, 
                    "NMI" = nmi)
  return(out)
}

library("SISIR")

context("Test that quality computation and plots for `SFCB` objects work as expected...")

data("truffles")
beta <- c(0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0)

test_that("quality computation works as expected without selection.", {
  expected_outputs <- c("dendro", "groups", "summaries", "mse", "importances",
                        "computational.times", "call", "truth",
                        "quality", "threshold")
  
  out1 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls")
  expect_named(quality(out1, beta, threshold = 0.001), expected_outputs)
  
  out2 <- sfcb(rainfall, truffles, group.method = "cclustofvar", 
               summary.method = "basics")
  expect_named(quality(out2, beta, threshold = 0.001), expected_outputs)
  
  out3 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", range.at = c(5, 7))
  expect_named(quality(out3, beta, threshold = 0.001), expected_outputs)
  
  out4 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "basics", range.at = c(5, 7))
  expect_named(quality(out4, beta, threshold = 0.01), expected_outputs)
})

test_that("quality computation works as expected with selection.", {
  expected_outputs <- c("dendro", "groups", "summaries", "selected", "mse", 
                        "importances", "computational.times", "call", "truth",
                        "quality")
  
  out4 <- sfcb(rainfall, truffles, group.method = "adjclust",
               summary.method = "pls", selection.method = "relief")
  expect_named(quality(out4, beta), expected_outputs)
  
  out5 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", selection.method = "relief", 
               range.at = c(5, 7))
  expect_named(quality(out5, beta), expected_outputs)
  
  out6 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "basics", selection.method = "relief")
  expect_named(quality(out6, beta), expected_outputs)
  
  out7 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "basics", selection.method = "relief", 
               range.at = c(5, 7))
  expect_named(quality(out7, beta), expected_outputs)
  
  out8 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", selection.method = "relief", 
               range.at = c(5, 12))
  expect_named(quality(out8, beta), expected_outputs)
  expect_named(quality(out8, beta, threshold = 0.01), 
               c(expected_outputs, "threshold"))
})

test_that("quality graphics works as expected.", {
  out1 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls")
  out1b <- quality(out1, beta, threshold = 0.001)
  p <- plot(out1b, plot.type = "quality", quality.crit = "mse")
  expect_s3_class(p, "ggplot")
  p <- plot(out1b, plot.type = "quality", quality.crit = "time")
  expect_s3_class(p, "ggplot")
  p <- plot(out1b, plot.type = "quality", quality.crit = "ARI")
  expect_s3_class(p, "ggplot")
  p <- plot(out1b, plot.type = "quality", quality.crit = "NMI")
  expect_s3_class(p, "ggplot")
  p <- plot(out1b, plot.type = "quality", quality.crit = c("mse", "NMI"))
  expect_s3_class(p, "ggplot")
  p <- plot(out1b, plot.type = "quality", 
            quality.crit = c("Precision", "Recall"))
  expect_s3_class(p, "ggplot")
  
  out2 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", range.at = c(5, 7))
  out2b <- quality(out2, beta, threshold = 0.001)
  p <- plot(out2b, plot.type = "quality", quality.crit = "mse")
  expect_s3_class(p, "ggplot")
  p <- plot(out2b, plot.type = "quality", quality.crit = "time")
  expect_s3_class(p, "ggplot")
  p <- plot(out2b, plot.type = "quality", quality.crit = "NMI")
  expect_s3_class(p, "ggplot")
  p <- plot(out2b, plot.type = "quality", quality.crit = c("mse", "ARI"))
  expect_s3_class(p, "ggplot")
  p <- plot(out2b, plot.type = "quality", 
            quality.crit = c("Precision", "Recall"))
  expect_s3_class(p, "ggplot")
  
  out3 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", selection.method = "relief", 
               range.at = c(5, 12))
  out3b <- extract_at(out3, c(9, 11:12))
  out3c <- quality(out3b, beta, threshold = 0.01)
  p <- plot(out3c, plot.type = "quality", quality.crit = "mse")
  expect_s3_class(p, "ggplot")
  p <- plot(out3c, plot.type = "quality", quality.crit = "ARI")
  expect_s3_class(p, "ggplot")
  p <- plot(out3c, plot.type = "quality", quality.crit = c("mse", "NMI"))
  expect_s3_class(p, "ggplot")
  p <- plot(out3c, plot.type = "quality", 
            quality.crit = c("Precision", "Recall"))
  expect_s3_class(p, "ggplot")
})

test_that("quality computation returns errors as expected.", {
  out1 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls")
  expect_error({ quality(out1, c(0, 1), threshold = 0.001) },
               "'ground_truth' must have a length identical to initial number",
               fixed = FALSE)
  expect_error({ quality(out1, beta, threshold = "A") },
               "'threshold' must be a positive number or NULL.",
               fixed = FALSE)
  expect_error({ quality(out1, beta, threshold = -3) },
               "'threshold' must be a positive number or NULL.",
               fixed = FALSE)
  expect_error({ quality(out1, beta) },
               "No selected interval in this 'SFCB' object and no 'threshold'",
               fixed = FALSE)
})

test_that("quality graphics return errors as expected.", {
  out1 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls")
  out1b <- quality(out1, beta, threshold = 0.001)
  expect_error({ plot(out1b, plot.type = "quality", 
                      quality.crit = c("mse", "time")) },
               "'time' is a valid quality criterion to plot only taken alone.",
               fixed = FALSE)
  expect_error({plot(out1, plot.type = "quality", quality.crit = "Precision")},
               "'quality.crit' must be a vector with length at most 2 in",
               fixed = FALSE)
  expect_error({plot(out1, plot.type = "quality", 
                     quality.crit = c("mse", "time", "Precision"))},
               "'quality.crit' must be a vector with length at most 2 in",
               fixed = FALSE)
  expect_error({ plot(out1, plot.type = "quality", quality.crit = "AA") },
               "'quality.crit' must be a vector with length at most 2 in",
               fixed = FALSE)
  
  out2 <- sfcb(rainfall, truffles, group.method = "adjclust", 
               summary.method = "pls", keep.time = FALSE)
  expect_error({ plot(out2, plot.type = "quality", quality.crit = "time") },
               "'quality.crit' must be a vector with length at most 2 in",
               fixed = FALSE)
})

#' @import ggplot2
#' @importFrom reshape2 melt
#' @import dendextend
#' @importFrom RColorBrewer brewer.pal
#' @importFrom stats as.dendrogram na.omit
#' @importFrom graphics abline

plot_dendrogram <- function(x) {
  
  dendro <- as.dendrogram(x$dendro)
  nlev <- dendextend::nleaves(dendro)
  labs <- labels(dendro)
  dendro <- set_labels(dendro, rep("", length(labs)))
  dendro %>% plot(leaflab = "none") 
  
  all_at <- unique(x$mse$clust)
  nrange <- length(x$groups)
  kmax <- max(all_at)
  basics <- ncol(x$importances[[1]]) != (nrow(x$mse) / length(all_at))
  if (basics) {
    all_groups <- rep(names(x$groups), 2)
    all_types <- rep(c("mean", "sd"), each = nrange)
  }
  
  all_h <- rev(x$dendro$height)
  
  # rectangles or lines
  if (all(diff(x$dendro$height) >= 0)) {
    dendro %>% rect.dendrogram(k = kmax)
  } else {
    message(paste("Reversals detected in the dendrogram. Rectangles are not",
                  "relevant and thus, they are not displayed."))
  }
  
  # horizontal lines ('range_at')
  if (nrange > 1) {
    for (k in setdiff(all_at, max(all_at))) {
      abline(h = (all_h[k] + all_h[k-1]) / 2, col = "darkred", lwd = 2)
    }
  }
  
  if (!basics) {
    if (nrange > 3) {
      cutb <- 3
      message(paste("Only the first 3 selections are represented below. If ",
                    "you want to display others, please use `extract_at`",
                    "before plotting."))
    } else cutb <- nrange
  } else {
    if (nrange > 2) {
      cutb <- 2
      message(paste("Only the first 2 selections are represented below. If",
                    "you want to display others, please use `extract_at`",
                    "before plotting."))
    } else cutb <- nrange
    all_types_c <- rep(c("mean", "sd"), each = cutb)
    all_groups_c <- rep(names(x$groups)[1:cutb], 2)
  }
  
  # bars
  if ("selected" %in% names(x)) { # case 'selected'
    values <- replicate(cutb, rep("grey", length(labs)), simplify = FALSE)
    if (!basics) {
      values <- mapply(function(a, b) {
        names(a) <- labs; a[b] <- "darkred"; return(a)
        }, values, x$selected[1:cutb])
      bar_labs <- paste("#int.:", names(x$groups)[1:cutb], "- sel:")
    } else {
      values_mean <- mapply(function(a, b) {
        names(a) <- labs; a[b$variable[b$"in.mean"]] <- "darkred"; return(a)
      }, values, x$selected[1:cutb])
      values_sd <- mapply(function(a, b) {
        names(a) <- labs; a[b$variable[b$"in.sd"]] <- "darkred"; return(a)
      }, values, x$selected[1:cutb])
      values <- cbind(values_mean, values_sd)
      bar_labs <- paste(all_groups_c, "/", all_types_c, "sel.")
    }
  } else {
    best <- tapply(x$mse$mse, x$mse$clust, which.min)
    if (!basics) {
      values <- mapply(function(a, b) return(a[, b]), 
                       x$importances[1:cutb], best[1:cutb])
      values <- apply(values, 2, cut, breaks = 9, labels = FALSE)
      values <- apply(values, 2, function(acol) brewer.pal(9, "YlOrRd")[acol])
      bar_labs <- paste("#int.:", names(x$groups)[1:cutb], "- imp.")
    } else {
      bestm <- paste0("mean", best)
      bestsd <- paste0("sd", best)
      values_m <- mapply(function(a, b) return(a[, b]), 
                         x$importances[1:cutb], bestm[1:cutb])
      values_m <- apply(values_m, 2, cut, breaks = 9, labels = FALSE)
      values_m <- apply(values_m, 2, 
                        function(acol) brewer.pal(9, "YlOrRd")[acol])
      values_sd <- mapply(function(a, b) return(a[, b]), 
                          x$importances[1:cutb], bestsd[1:cutb])
      values_sd <- apply(values_sd, 2, cut, breaks = 9, labels = FALSE)
      values_sd <- apply(values_sd, 2, 
                         function(acol) brewer.pal(9, "YlOrRd")[acol])
      values_sd[is.na(values_sd)] <- "black"
      values <- cbind(values_m, values_sd)
      bar_labs <- paste(all_groups_c, "/", all_types_c, "- imp.")
    }
  }
  # bars
  colored_bars(colors = values, dend = dendro, rowLabels = bar_labs)

  return(invisible(NULL))
}

plot_importance <- function(x, shape.imp) {
  lobj <- length(x$importances)
  nrepeats <- nrow(x$mse) / lobj
  basics <- ncol(x$importances[[1]]) != nrepeats
  
  suppressMessages({ df <- reshape2::melt(x$importances) })
  if (!basics) { # not basics
    names(df) <- c("variable", "repeats", "importance", "at")
  } else { # basics
    df$type <- rep("mean", nrow(df))
    df$type[grep("sd", df$variable)] <- "sd"
    names(df) <- c("repeats", "importance", "at", "type")
  }
  
  if (shape.imp == "boxplot") {
    p <- ggplot(df, aes(x = .data$repeats, y = .data$importance)) + 
      geom_boxplot() + theme_bw()
  } else if (shape.imp == "histogram") {
    p <- ggplot(df, aes(x = .data$importance)) + geom_histogram(bins = 30) + 
      theme_bw()
  }
  
  if (lobj > 1) {
    if (!basics | shape.imp == "boxplot") {
      p <- p + facet_wrap(~ at)
    } else {
      p <- p + facet_grid(type ~ at)
    }
  } else if (shape.imp == "histogram" & basics) { # basics
    p <- p + facet_wrap(~ type)
  }
  
  return(p)
}

plot_selection <- function(x, sel.type, threshold) {
  is_selected <- "selected" %in% names(x)
  
  if (threshold != "none") {
    if (!is.numeric(threshold)) 
      stop("'threshold' must be numeric.", call. = FALSE)
    
    if (is_selected) 
      stop(paste("A selection method has already been used: 'threshold' must",
                 "be 'none'."),
           call. = FALSE)
           
    if (sel.type != "selection") {
      warning(paste("A 'threshold' has passed to the plot function while",
                    "'sel.type' is not 'selection'. Automatically switching it",
                    "to 'selection'."),
              call. = FALSE)
      sel.type <- "selection"
    }
  }
  
  if (sel.type == "selection" && !is_selected && threshold == "none")
    stop(paste("Variable selection was not used on this result. Choose",
               "sel.type = 'importance'"),
         call. = FALSE)
  
  lobj <- length(x$importances)
  nrepeats <- nrow(x$mse) / lobj
  basics <- ncol(x$importances[[1]]) != nrepeats
  var_names <- names(x$groups[[1]])
  
  if (threshold != "none") {
    if (!basics) { # not basics, 'at'
      best <- tapply(x$mse$mse, x$mse$clust, which.min)
      x$selected <- mapply(function(a, b) { rownames(a)[a[, b] > threshold] }, 
                           x$importances, best, SIMPLIFY = FALSE)
    } else {
      best <- tapply(x$mse$mse, x$mse$clust, which.min)
      x$selected <- mapply(function(a, b) {
        bmean <- paste0("mean", b)
        bmean <- rownames(a)[a[, bmean] > threshold]
        bsd <- paste0("sd", b)
        bsd <- rownames(a)[a[, bsd] > threshold]
        bsd <- na.omit(bsd)
        variables <- unique(c(bmean, bsd))
        variables <- var_names[var_names %in% variables]
        out <- data.frame("variable" = variables,
                          "in.mean" = variables %in% bmean,
                          "in.sd" = variables %in% bsd)
        return(out)
      }, x$importances, best, SIMPLIFY = FALSE)
    }
  }
  
  if (sel.type == "selection") { # not basics, 'at'
    if (lobj == 1 & !basics) {
      dfline <- make_line_df(x$selected[[1]], var_names)
      dfrect <- make_rect_df(x$selected[[1]], var_names)
      
      p <- ggplot() +
        geom_rect(data = dfrect, 
                  aes(xmin = .data$xstart, ymin = .data$ystart, 
                      xmax = .data$xend, ymax = .data$yend), 
                  alpha = 0.2)
    } else if (!basics) { # not basics, 'range_at'
      all_levels <- seq(0.7, 1, length.out = length(x$selected))
      all_levels2 <- seq(0, 0.3, length.out = length(x$selected))
      dfline <- mapply(function(a, b, d) {
        make_line_df(a, var_names = var_names, level = b, level2 = d)
      }, x$selected, all_levels, all_levels2, SIMPLIFY = FALSE)
      
      all_at <- sapply(dfline, nrow)
      all_at <- rep(names(dfline), all_at)
      dfline <- data.frame(Reduce(rbind, dfline), "at" = all_at)
      
      p <- ggplot(data = dfline, aes(group = .data$at, colour = .data$at))
    } else if (lobj == 1) { # basics, 'at'
      sel_mean <- x$selected[[1]]$variable[x$selected[[1]]$"in.mean"]
      dfline_mean <- make_line_df(sel_mean, var_names, level2 = 0.2)
      sel_sd <- x$selected[[1]]$variable[x$selected[[1]]$"in.sd"]
      dfline_sd <- make_line_df(sel_sd, var_names, level = 0.8)
      dfline <- rbind(dfline_mean, dfline_sd)
      dfline$type <- c(rep("mean", nrow(dfline_mean)),
                       rep("sd", nrow(dfline_sd)))
      
      p <- ggplot(data = dfline, aes(group = .data$type, colour = .data$type))
    } else { # basics, 'range_at'
      dfmean <- lapply(x$selected, 
                       function(alist) alist$variable[alist$"in.mean"])
      names(dfmean) <- paste0("mean", names(dfmean))
      dfsd <- lapply(x$selected, function(alist) alist$variable[alist$"in.sd"])
      names(dfsd) <- paste0("sd", names(dfmean))
      df <- c(dfmean, dfsd)
      
      all_levels <- seq(0.7, 1, length.out = length(df))
      all_levels2 <- seq(0, 0.3, length.out = length(df))
      dfline <- mapply(function(a, b, d) {
        make_line_df(a, var_names = var_names, level = b, level2 = d)
      }, df, all_levels, all_levels2, SIMPLIFY = FALSE)
      
      all_at <- sapply(dfline, nrow)
      all_at <- rep(names(dfline), all_at)
      dfline <- data.frame(Reduce(rbind, dfline), "at" = all_at)
      
      p <- ggplot(data = dfline, aes(group = .data$at, colour = .data$at))
    }
    
     p <- p +
      geom_segment(data = dfline, 
                   aes(x = .data$xstart, y = .data$ystart, xend = .data$xend, 
                       yend = .data$yend), 
                   size = 1) +
      theme_bw() + xlim(1, length(var_names)) + 
      scale_y_continuous(breaks = c(0, 1), minor_breaks = c(0, 1), 
                         labels = c("not selected", "selected"),
                         limits = c(0, 1.23)) +
      theme(axis.title.y = element_blank(), axis.title.x = element_blank())
     if (lobj == 1 & basics) { # basics, 'at'
       p <- p + guides(colour = guide_legend(title = "for summary..."))
     } else if (!basics & lobj > 1) { # not basics, 'range_at'
       p <- p + guides(colour = guide_legend(title = "# intervals"))
     } else if (lobj > 1) { # basics, 'range_at'
       p <- p + guides(colour = guide_legend(title = "summary + # interv."))
     }
    
  } else { # sel.type == "importance"
    if (lobj == 1 & !basics) {
      imp_names <- rownames(x$importances[[1]])
      impmean <- data.frame("x" = match(imp_names, var_names),
                            "mean" = apply(x$importances[[1]], 1, mean))
      impmin <- data.frame("x" = match(imp_names, var_names),
                           "min" = apply(x$importances[[1]], 1, min))
      impmax <- data.frame("x" = match(imp_names, var_names),
                           "max" = apply(x$importances[[1]], 1, max))
      df <- merge(impmean, impmin)
      df <- merge(df, impmax)
      df <- fill_na(df, c("mean", "min", "max"))
      p <- ggplot(df, aes(x = .data$x))
    } else if (!basics) {
      impmean <- compute_sumimp(x$importances, "mean", var_names)
      impmin <- compute_sumimp(x$importances, "min", var_names)
      impmax <- compute_sumimp(x$importances, "max", var_names)
      df <- merge(impmean, impmin)
      df <- merge(df, impmax)
      df <- fill_na(df, c("mean", "min", "max"))
      all_at <- as.character(sort(unique(df$at)))
      df$at <- factor(df$at, levels = all_at, ordered = TRUE)
      p <- ggplot(df, aes(x = .data$x, group = .data$at, colour = .data$at, 
                          fill = .data$at))
    } else if (lobj == 1) {
      impmean <- compute_sumimp1_basics(x$importances[[1]], "mean", var_names)
      impmin <- compute_sumimp1_basics(x$importances[[1]], "min", var_names)
      impmax <- compute_sumimp1_basics(x$importances[[1]], "max", var_names)
      df <- merge(impmean, impmin)
      df <- merge(df, impmax)
      df <- fill_na(df, c("mean", "min", "max"))
      p <- ggplot(df, aes(x = .data$x, colour = .data$type, fill = .data$type, 
                          group = .data$type))
    } else {
      impmean <- compute_sumimp_basics(x$importances, "mean",var_names)
      impmin <- compute_sumimp_basics(x$importances, "min", var_names)
      impmax <- compute_sumimp_basics(x$importances, "max", var_names)
      df <- merge(impmean, impmin)
      df <- merge(df, impmax)
      df <- fill_na(df, c("mean", "min", "max"))
      p <- ggplot(df, aes(x = .data$x, group = .data$at, colour = .data$at, 
                          fill = .data$at))
    }
    
    p <- p + geom_ribbon(aes(ymin = .data$min, ymax = .data$max), alpha = 0.1) + 
      geom_line(aes(y = mean)) + theme_bw() + 
      ylab("variable importance") +
      theme(axis.title.x = element_blank(), axis.text.x = element_blank(),
            axis.ticks.x = element_blank())
  }
  
  return(p)
}

plot_quality <- function(x, quality.crit) {
  
  valid_criteria <- "mse"
  if ("computational.times" %in% names(x))  {
    valid_criteria <- c(valid_criteria, "time")
  }
  if ("quality" %in% names(x)) {
    valid_criteria <- c(valid_criteria, "Precision", "Recall", "ARI", "NMI")
  }
  
  crit_ok <- all(sapply(quality.crit, function(cc) cc %in% valid_criteria))
  if (!crit_ok || length(quality.crit) > 2) {
    stop(paste0("'quality.crit' must be a vector with length at most 2 in ",
                paste(valid_criteria, collapse = ", "), "."),
         call. = FALSE)
  }
  
  if (length(quality.crit) == 2 && "time" %in% quality.crit) {
    stop("'time' is a valid quality criterion to plot only taken alone.",
         call. = FALSE)
  }
  
  if ("time" %in% quality.crit) {
    df <- data.frame("criterion" = c(unname(x$"computational.times")),
                     "step" = names(x$"computational.times"))
    df$step <- factor(df$step, levels = names(x$"computational.times"),
                      ordered = TRUE)
    p <- ggplot(df, aes(fill = .data$step, y = .data$criterion, x = 1)) + 
      geom_bar(stat = "identity") + theme_bw() + xlim(0, 2) +
      ylab("computational time (s)") +
      theme(axis.title.x = element_blank(), axis.ticks.x = element_blank(),
            axis.text.x = element_blank()) +
      scale_fill_brewer(type = "qual", palette = 7)
    return(p)
  }
  
  if (length(quality.crit) == 1) {
    # build data frame
    if (quality.crit == "mse") {
      df <- data.frame("criterion" = x$mse$mse, "at" = x$mse$clust)
      ylimits <- c(0, max(df$criterion))
    } else {
      df <- data.frame("criterion" = x$quality[, quality.crit], 
                       "at" = as.numeric(x$quality$clust))
      if (quality.crit == "ARI") {
        ylimits <- c(-1, 1)
      } else ylimits <- c(0, 1)
    }
    
    # make plot
    p <- ggplot(df, aes(x = .data$at, y = .data$criterion)) + 
      geom_jitter(width=0.2, height = 0) + theme_bw() + 
      xlab("number of intervals") + ylab(quality.crit) + ylim(ylimits) +
      scale_x_continuous(breaks = unique(df$at), 
                         limits = c(min(df$at) - 0.5, max(df$at + 0.5)))
    
  } else {
    if ("mse" %in% quality.crit) {
      quality.crit <- setdiff(quality.crit, "mse")
      df <- data.frame("x" = x$quality[, quality.crit], "y" = x$mse$mse, 
                       "at" = as.factor(x$quality$clust))
      quality_names <- c(quality.crit, "mse")
      if (quality.crit == "ARI") {
        xlimits <- c(-1, 1)
      } else xlimits <- c(0, 1)
      ylimits <- c(0, max(df$y))
    } else {
      df <- data.frame("x" = x$quality[, quality.crit[1]], 
                       "y" = x$quality[, quality.crit[2]], 
                       "at" = as.factor(x$quality$clust))
      quality_names <- quality.crit
      if (quality.crit[1] == "ARI") {
        xlimits <- c(-1, 1)
      } else xlimits <- c(0, 1)
      if (quality.crit[2] == "ARI") {
        ylimits <- c(-1, 1)
      } else ylimits <- c(0, 1)
    }
    p <- ggplot(df, aes(x = .data$x, y = .data$y, colour = .data$at)) + 
      geom_point() + theme_bw() + xlab(quality_names[1]) + 
      ylab(quality_names[2]) + xlim(xlimits) + ylim(ylimits) + 
      scale_colour_discrete(name = "# intervals")
    if (length(unique(df$at)) > 20) {
      p <- p + theme(legend.position = "none")
    }
  }
  
  return(p)
}

compute_sumimp <- function(importances, func_name, var_names) {
  FUN <- eval(func_name)
  impsummary <- lapply(importances, function(alist) apply(alist, 1, FUN))
  impsummary <- lapply(impsummary, function(alist) {
    out <- rep(NA, length(var_names))
    names(out) <- var_names
    out[match(names(alist), var_names)] <- alist
    return(out)
  })
  impsummary <- Reduce(cbind, impsummary)
  rownames(impsummary) <- match(rownames(impsummary), var_names)
  colnames(impsummary) <- names(importances)
  impsummary <- melt(impsummary)
  names(impsummary) <- c("x", "at", func_name)
  return(impsummary)
}

compute_sumimp1_basics <- function(importances, func_name, var_names) {
  FUN <- eval(func_name)
  
  selected <- grep("mean", names(importances))
  importances_mean <- importances[, selected]
  selected <- grep("sd", names(importances))
  importances_sd <- importances[, selected]
  importances <- list(importances_mean, importances_sd)
  names(importances) <- c("mean", "sd")
  
  impsummary <- lapply(importances, function(alist) 
    data.frame("x" = match(rownames(alist), var_names),
               "importance" = apply(alist, 1, FUN, na.rm = TRUE)))
  impsummary <- reshape2::melt(impsummary, id = "x")
  names(impsummary) <- c("x", "variable", func_name, "type")
  
  return(impsummary)
}

compute_sumimp_basics <- function(importances, func_name, var_names) {
  FUN <- eval(func_name)
  importances_mean <- lapply(importances, function(alist) {
    selected <- grep("mean", names(alist))
    return(alist[, selected])
  })
  importances_sd <- lapply(importances, function(alist) {
    selected <- grep("sd", names(alist))
    return(alist[, selected])
  })
  importances <- c(importances_mean, importances_sd)
  imp_names <- rep(c("mean", "sd"), each = length(importances) / 2)
  names(importances) <- paste0(imp_names, names(importances))
  impsummary <- lapply(importances, function(alist) 
    apply(alist, 1, FUN, na.rm = TRUE))
  impsummary <- lapply(impsummary, function(alist) {
    out <- rep(NA, length(var_names))
    names(out) <- var_names
    out[match(names(alist), var_names)] <- alist
    return(out)
  })
  impsummary <- Reduce(cbind, impsummary)
  rownames(impsummary) <- match(rownames(impsummary), var_names)
  colnames(impsummary) <- names(importances)
  impsummary <- melt(impsummary)
  names(impsummary) <- c("x", "at", func_name)
  return(impsummary)
}

fill_na <- function(dataset, var) {
  out <- sapply(seq_along(var), function(ind) {
    outcol <- dataset[, var[ind]]
    outcol[is.na(outcol)] <- 0
    return(outcol)
  })
  dataset[, var] <- out
  out <- sapply(seq_along(var), function(ind) {
    outcol <- dataset[, var[ind]]
    outcol[outcol == Inf] <- 0
    return(outcol)
  })
  dataset[, var] <- out
  return(dataset)
}

make_line_df <- function(selected, var_names, level = 1, level2 = 0) {
  which_sel <- match(selected, var_names)
  if (length(which_sel) > 0 & length(which_sel) < length(var_names)) {
    diff_which <- diff(which_sel)
    starts <- which(diff_which > 1) + 1
    starts <- c(which_sel[1], which_sel[starts])
    ends <- which(diff_which > 1)
    ends <- c(which_sel[ends], which_sel[length(which_sel)])
    df <- data.frame("xstart" = starts, "ystart" = rep(level, length(starts)),
                     "xend" = ends, "yend" = rep(level, length(starts)))
    which_sel <- setdiff(seq_along(var_names), which_sel)
    diff_which <- diff(which_sel)
    starts <- which(diff_which > 1) + 1
    starts <- c(which_sel[1], which_sel[starts])
    ends <- which(diff_which > 1)
    ends <- c(which_sel[ends], which_sel[length(which_sel)])
    df2 <- data.frame("xstart" = starts, "ystart" = rep(level2, length(starts)),
                      "xend" = ends, "yend" = rep(level2, length(starts)))
    df <- rbind(df, df2)
  } else if (length(which_sel) == 0) {
    df <- data.frame("xstart" = 1, "ystart" = 0, "xend" = length(var_names),
                     "yend" = 0)
  } else {
    df <- data.frame("xstart" = 1, "ystart" = level, "xend" = length(var_names),
                     "yend" = level)
  }
  return(df)
}

make_rect_df <- function(selected, var_names) {
  which_sel <- match(selected, var_names)
  diff_which <- diff(which_sel)
  starts <- which(diff_which > 1) + 1
  starts <- c(which_sel[1], which_sel[starts])
  ends <- which(diff_which > 1)
  ends <- c(which_sel[ends], which_sel[length(which_sel)])
  df_rect <- data.frame("xstart" = starts, "ystart" = rep(0, length(starts)), 
                        "xend" = ends, "yend" = rep(1, length(starts)))

  return(df_rect)
}

#' @import magrittr
#' @importFrom dplyr mutate filter arrange select summarise ungroup inner_join
#' @importFrom tidyselect starts_with
#' @importFrom tidyr gather spread
#' @importFrom purrr map_dfc
#' @importFrom mixOmics pls
#' @importFrom stats sd as.hclust
#' @importFrom rlang .data

summary_pls <- function(dataset, target, groups) {
  # Description #####
  # returns a list of datasets with the same rows than 'dataset' and with a
  # number of columns corresponding to the different values of groups
  # (containing the PLS with respect to the target of the columns in 'dataset'
  # that are in the same group)
  # Inputs #####
  # dataset: a data.frame with observations in rows and variables in columns
  # target: target variable to predict
  # groups: a list with a hierarchy of groups
  # Outputs #####
  # a list where each entry is a dataset where the summary has been performed
  # for the groups
  
  p <- ncol(dataset)
  colnames(dataset) <- paste0("Var", 1:p)
  
  summary_data <- lapply(groups, function(cur_clust) {
    cur_data <- dataset %>% dplyr::mutate("sim" = 1:nrow(dataset)) %>% 
      tidyr::gather("Var", "value", - .data$sim)
    var_number <- as.numeric(gsub("Var", "", cur_data$Var))
    cur_data$clust <- cur_clust[var_number]
    
    pls_summary <- purrr::map_dfc(unique(cur_clust), compute_pls_summary, 
                                  dataset = cur_data, target = target)
    
    return(pls_summary)
  })
  
  return(summary_data)
}

compute_pls_summary <- function(aclust, dataset, target) {
  tmp <- dataset %>% dplyr::filter(.data$clust == aclust) %>% 
    tidyr::spread(.data$Var, .data$value) %>% dplyr::arrange(.data$sim) %>% 
    dplyr::select(tidyselect::starts_with("Var"))
  if (length(unique(dataset$Var[dataset$clust == aclust])) > 1) {
    out <- mixOmics::pls(tmp, target, ncomp = 1)
    return(unlist(out$variates$X))
  } 
  
  return(unlist(tmp))
}

summary_basics <- function(dataset, groups) {
  # Description #####
  # returns a list of datasets with the same rows than 'dataset' and with a
  # number of columns corresponding to the different values of groups
  # (containing the mean and sd of the columns within the same group)
  # Inputs #####
  # dataset: a data.frame with observations in rows and variables in columns
  # groups: a list with a hierarchy of groups
  # Outputs #####
  # a list where each entry is a dataset where the summary has been performed
  # for the groups
  
  p <- ncol(dataset)
  colnames(dataset) <- paste0("Var", 1:p)
  
  summary_data <- lapply(groups, function(cur_clust) {
    cur_data <- dataset %>% dplyr::mutate("sim" = 1:nrow(dataset)) %>% 
      tidyr::gather("Var", "value", - .data$sim)
    var_number <- as.numeric(gsub("Var", "", cur_data$Var))
    cur_data$clust <- cur_clust[var_number]
    
    mean_summary <- cur_data %>% dplyr::group_by(.data$sim, .data$clust) %>%
      dplyr::summarise(VarMean = mean(.data$value)) %>%
      tidyr::spread(.data$clust, value = .data$VarMean, sep = "mean") %>%
      dplyr::ungroup()
    
    sd_summary <- cur_data %>% dplyr::group_by(.data$sim, .data$clust) %>%
      dplyr::summarise(VarSD = sd(.data$value)) %>%
      tidyr::spread(.data$clust, value = .data$VarSD, sep = "sd") %>%
      dplyr::ungroup()
    
    cur_data <- dplyr::inner_join(mean_summary, sd_summary, by = "sim") %>%
      dplyr::select(- .data$sim)
    
    # remove column with NAs (for standard deviations)
    contains_na <- colSums(apply(cur_data, 2, is.na))
    cur_data <- cur_data[, contains_na == 0]
    
    return(cur_data)
  })
  
  return(summary_data)
}

summary_cclustofvar <- function(dataset, groups) {
  # Description #####
  # returns a list of datasets with the same rows than 'dataset' and with a
  # number of columns corresponding to the different values of groups
  # (containing the cov consensus variable)
  # Inputs #####
  # dataset: a data.frame with observations in rows and variables in columns
  # groups: a list with a hierarchy of groups
  # Outputs #####
  # a list where each entry is a dataset where the summary has been performed
  # for the groups
  
  p <- ncol(dataset)
  colnames(dataset) <- paste0("Var", 1:p)
  
  summary_data <- lapply(groups, function(cur_clust) {
    cur_data <- dataset %>% dplyr::mutate("sim" = 1:nrow(dataset)) %>% 
      tidyr::gather("Var", "value", - .data$sim)
    var_number <- as.numeric(gsub("Var", "", cur_data$Var))
    cur_data$clust <- cur_clust[var_number]
    
    cov_summary <- purrr::map_dfc(unique(cur_clust), compute_cov_summary, 
                                  dataset = cur_data)
    
    return(cov_summary)
  })
  
  return(summary_data)
}

compute_cov_summary <- function(aclust, dataset) {
  tmp <- dataset %>% dplyr::filter(.data$clust == aclust) %>% 
    tidyr::spread(.data$Var, .data$value) %>% dplyr::arrange(.data$sim) %>% 
    dplyr::select(starts_with("Var"))
  if (length(unique(dataset$Var[dataset$clust == aclust])) > 1) {
    out <- svd(tmp, 1, 1)
    n <- nrow(tmp)
    out <- sqrt(n) * out$d[1] * out$u[ ,1]
    return(out)
  }
  
  return(unlist(tmp))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sparseSIR.R
\name{sparseRes}
\alias{sparseRes}
\alias{summary.sparseRes}
\alias{print.sparseRes}
\alias{sparseRes-class}
\title{Print sparseRes object}
\usage{
\method{summary}{sparseRes}(object, ...)

\method{print}{sparseRes}(x, ...)
}
\arguments{
\item{object}{a \code{sparseRes} object}

\item{...}{not used}

\item{x}{a \code{sparseRes} object}
}
\description{
Print a summary of the result of \code{\link{sparseSIR}} (
\code{sparseRes} object)
}
\seealso{
\code{\link{sparseSIR}}
}
\author{
{Victor Picheny, \email{victor.picheny@inrae.fr}\cr
Remi Servien, \email{remi.servien@inrae.fr}\cr
Nathalie Vialaneix, \email{nathalie.vialaneix@inra.fr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sparseSIR.R
\name{project}
\alias{project}
\alias{project.sparseRes}
\title{sparse SIR}
\usage{
\method{project}{sparseRes}(object)

project(object)
}
\arguments{
\item{object}{an object of class \code{sparseRes} as obtained from the 
function \code{\link{sparseSIR}}}
}
\value{
a matrix of dimension n x d with the projection of the observations
on the d dimensions of the sparse EDR space
}
\description{
\code{project} performs the projection on the sparse EDR space (as obtained
by the \code{\link[glmnet]{glmnet}})
}
\details{
The projection is obtained by the function 
\code{\link[glmnet]{predict.glmnet}}.
}
\examples{
set.seed(1140)
tsteps <- seq(0, 1, length = 200)
nsim <- 100
simulate_bm <- function() return(c(0, cumsum(rnorm(length(tsteps)-1, sd=1))))
x <- t(replicate(nsim, simulate_bm()))
beta <- cbind(sin(tsteps*3*pi/2), sin(tsteps*5*pi/2))
beta[((tsteps < 0.2) | (tsteps > 0.5)), 1] <- 0
beta[((tsteps < 0.6) | (tsteps > 0.75)), 2] <- 0
y <- log(abs(x \%*\% beta[ ,1]) + 1) + sqrt(abs(x \%*\% beta[ ,2]))
y <- y + rnorm(nsim, sd = 0.1)
\donttest{
res_ridge <- ridgeSIR(x, y, H = 10, d = 2)
res_sparse <- sparseSIR(res_ridge, rep(1, ncol(x)))
proj_data <- project(res_sparse)
}

}
\references{
{Picheny, V., Servien, R. and Villa-Vialaneix, N. (2016) 
Interpretable sparse SIR for digitized functional data.
\emph{Statistics and Computing}, \strong{29}(2), 255--267.}
}
\seealso{
\code{\link{sparseSIR}}
}
\author{
{Victor Picheny, \email{victor.picheny@inrae.fr}\cr
Remi Servien, \email{remi.servien@inrae.fr}\cr
Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sfcb_methods.R
\name{SFCB-class}
\alias{SFCB-class}
\alias{summary.SFCB}
\alias{print.SFCB}
\alias{plot.SFCB}
\alias{extract_at.SFCB}
\alias{quality.SFCB}
\alias{extract_at}
\alias{quality}
\title{Methods for SFCB objects}
\usage{
\method{summary}{SFCB}(object, ...)

\method{print}{SFCB}(x, ...)

\method{plot}{SFCB}(
  x,
  ...,
  plot.type = c("dendrogram", "selection", "importance", "quality"),
  sel.type = c("importance", "selection"),
  threshold = "none",
  shape.imp = c("boxplot", "histogram"),
  quality.crit = "mse"
)

extract_at(object, at)

quality(object, ground_truth, threshold = NULL)
}
\arguments{
\item{object}{a \code{SFCB} object}

\item{...}{not used}

\item{x}{a \code{SFCB} object}

\item{plot.type}{type of the plot. Default to \code{"dendrogram"} (see
Details)}

\item{sel.type}{when \code{plot.type == "selection"}, criterion on which to
base the selection. Default to \code{"importance"}}

\item{threshold}{numeric value. If not \code{NULL}, selection of variables to
compute qualities is based on a threshold of importance values
\code{extract_at}}

\item{shape.imp}{when \code{plot.type == "importance"}, type of plot to
represent the importance. Default to \code{"boxplot"}}

\item{quality.crit}{character vector (length 1 or 2) indicating one or two
quality criteria to display. The values have to be taken in \{\code{"mse"},
\code{"time"}, \code{"Precision"}, \code{"Recall"}, \code{"ARI"}, 
\code{"NMI"}\}. If \code{"time"} is chosen, it can not be associated with any
other criterion}

\item{at}{numeric vector. Set of the number of intervals to extract for}

\item{ground_truth}{numeric vector of ground truth. Target variables 
to compute qualities correspond to non-zero entries of this vector}
}
\description{
Print, plot, manipulate or compute quality for outputs of the
\code{\link{sfcb}} function (\code{SFCB} object)
}
\details{
The \code{plot} functions can be used in four different ways to 
extract information from the \code{SFCB} object: \itemize{
 \item \code{plot.type == "dendrogram"} displays the dendrogram obtained at
 the clustering step of the method. Depending on the cases, the dendrogram
 comes with additional information on clusters, variable selections and/or
 importance values;
 \item \code{plot.type == "selection"} displays either the evolution of the
 importance for the simulation with the best (smallest) MSE for each time
 step in the range of the functional predictor or the evolution of the
 selected intervals along the whole range of the functional prediction also 
 for the best MSE;
 \item \code{plot.type == "importance"} displays a summary of the importance
 values over the whole range of the functional predictor and for the 
 different experiments. This summary can take the form of a boxplot or of
 an histogram;
 \item \code{plot.type == "quality"} displays one or two quality distribution
 with respect to the different experiments and different number of intervals.
}
}
\examples{
data(truffles)
out1 <- sfcb(rainfall, truffles, group.method = "adjclust", 
             summary.method = "pls", selection.method = "relief")
summary(out1)

plot(out1)
plot(out1, plot.type = "selection")
plot(out1, plot.type = "importance")

out2 <- sfcb(rainfall, truffles, group.method = "adjclust", 
             summary.method = "basics", selection.method = "none",
             range.at = c(5, 7))
out3 <- extract_at(out2, at = 6)
summary(out3)

}
\references{
{Servien, R. and Vialaneix, N. (2023) A random forest approach 
for interval selection in functional regression. Preprint.}
}
\seealso{
\code{\link{sfcb}}
}
\author{
{Remi Servien, \email{remi.servien@inrae.fr}\cr
Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{truffles}
\alias{truffles}
\alias{rainfall}
\alias{beta}
\title{Dataset "Truffles"}
\format{
3 datasets are provided: \itemize{
  \item \code{rainfall}: a data frame with 15 columns (months from January 
  Year n to March Year n+1) and 25 rows (production years from 1924/1925 to 
  1948/1949). Data correspond to cumulated rainfall in mm;
  \item \code{truffles}: a vector with 25 values corresponding to the total 
  production (in kg) of truffles in the truffle patch of T. melanosporum de 
  Pernes-Les-Fontaines (Vaucluse, France);
  \item \code{beta}: 0/1 vector with 15 values indicated the months during 
  which the rainfall has the most important influence on the truffle 
  production, as provided by experts.
}
}
\description{
Yearly truffles production and corresponding monthly rainfall 
information of the Perigord black truffle in the Vaucluse (France) between
1924 and 1949.
}
\details{
This dataset has been made available by courtesy of the authors of 
the publication [Baragatti \emph{et al.}, 2019]. Meteorological data have 
been provided by Meteo France https://meteofrance.com (Orange meteorological 
station) and truffle production data are courtesy of the truffle patch.
}
\examples{
data(truffles)
summary(truffles)
plot(1:15, rainfall[1, ], type = "l", xlab = "month", ylab = "rainfall (mm)")
}
\references{
Baragatti M., Grollemund P.M., Montpied P., Dupouey J.L., Gravier J., Murat 
C., Le Tacon F. (2019) Influence of annual climatic variations, climate
changes, and sociological factors on the production of the Perigord black 
truffle (\emph{Tuber melanosporum Vittad.}) from 1903-1904 to 1988-1989 in 
the Vaucluse (France), \emph{Mycorrhiza}, \strong{29}(2), 113-125.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SFCB.R
\name{sfcb}
\alias{sfcb}
\title{sfcb}
\usage{
sfcb(
  X,
  Y,
  group.method = c("adjclust", "cclustofvar"),
  summary.method = c("pls", "basics", "cclustofvar"),
  selection.method = c("none", "boruta", "relief"),
  at = round(0.15 * ncol(X)),
  range.at = NULL,
  seed = NULL,
  repeats = 5,
  keep.time = TRUE,
  verbose = TRUE,
  parallel = FALSE
)
}
\arguments{
\item{X}{input predictors (matrix or data.frame)}

\item{Y}{target variable (vector whose length is equal to the number of rows
in X)}

\item{group.method}{group method. Default to \code{"adjclust"}}

\item{summary.method}{summary method. Default to \code{"pls"}}

\item{selection.method}{selection method. Default to \code{"none"} (no 
selection performed)}

\item{at}{number of groups targeted for output results (integer). Not used
when \code{range.at} is not \code{NULL}}

\item{range.at}{(vector of integer) sequence of the numbers of groups for
output results}

\item{seed}{random seed (integer)}

\item{repeats}{number of repeats for the final random forest computation}

\item{keep.time}{keep computational times for each step of the method? 
(logical; default to \code{TRUE})}

\item{verbose}{print messages? (logical; default to \code{TRUE})}

\item{parallel}{not implemented yet}
}
\value{
an object of class \code{"SFCB"} with elements: 
  \item{dendro}{a dendrogram corresponding to the method chosen in
  \code{group.method}}
  \item{groups}{a list of length \code{length(range.at)} (or of length 1 if
  \code{range.at == NULL}) that contains the clusterings of input variables
  for the selected group numbers}
  \item{summaries}{a list of the same length than \code{$groups} that 
  contains the summarized predictors according to the method chosen in 
  \code{summary.methods}}
  \item{selected}{a list of the same length than \code{$groups} that contains
  the names of the variable selected by \code{selection.method} if it is not
  equal to \code{"none"}}
  \item{mse}{a data.frame with \code{repeats} \eqn{\times}
  \code{length($groups)} rows that contains Mean Squared Errors of the 
  \code{repeats} random forests fitted for each number of groups}
  \item{importance}{a list of the same length than \code{$groups} that 
  contains a data.frame providing variable importances for the variables in
  selected groups in \code{repeats} columns (one for each iteration of the
  random forest method). When \code{summary.method == "basics"}, importance 
  for mean and sd are provided in separated columns, in which case, the 
  number of columns is equal to 2\code{repeats}}
  \item{computational.times}{a vector with 4 values corresponding to the
  computational times of (respectively) the group, summary, selection, and RF
  steps. Only if \code{keep.time == TRUE}}
  \item{call}{function call}
}
\description{
\code{sfcb} performs interval selection based on random forests
}
\examples{
data(truffles)
out1 <- sfcb(rainfall, truffles, group.method = "adjclust", 
             summary.method = "pls", selection.method = "relief")
out2 <- sfcb(rainfall, truffles, group.method = "adjclust", 
             summary.method = "basics", selection.method = "none",
             range.at = c(5, 7))
}
\references{
Servien, R. and Vialaneix, N. (2024) A random forest approach 
for interval selection in functional regression. \emph{Statistical Analysis
and Data Mining}, \strong{17}(4), e11705. \doi{10.1002/sam.11705}
}
\author{
Remi Servien, \email{remi.servien@inrae.fr}\cr
Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sparseSIR.R
\name{sparseSIR}
\alias{sparseSIR}
\title{sparse SIR}
\usage{
sparseSIR(
  object,
  inter_len,
  adaptive = FALSE,
  sel_prop = 0.05,
  parallel = FALSE,
  ncores = NULL
)
}
\arguments{
\item{object}{an object of class \code{ridgeRes} as obtained from the 
function \code{\link{ridgeSIR}}}

\item{inter_len}{(numeric) vector with interval lengths}

\item{adaptive}{should the function returns the list of strong zeros and non
strong zeros (logical). Default to FALSE}

\item{sel_prop}{used only when \code{adaptive = TRUE}. Fraction of the 
coefficients that will be considered as strong zeros and strong non zeros.
Default to 0.05}

\item{parallel}{whether the computation should be performed in parallel or
not. Logical. Default is FALSE}

\item{ncores}{number of cores to use if \code{parallel = TRUE}. If left to 
NULL, all available cores minus one are used}
}
\value{
S3 object of class \code{sparseRes}: a list consisting of
\describe{
   \item{\code{sEDR}}{ the estimated EDR space (a p x d matrix)}
   \item{\code{alpha}}{ the estimated shrinkage coefficients (a vector having
   a length similar to \code{inter_len})}
   \item{\code{quality}}{ a vector with various qualities for the model (see
   Details)}
   \item{\code{adapt_res}}{ if \code{adaptive = TRUE}, a list of two vectors: 
   \describe{
     \item{\code{nonzeros}}{ indexes of variables that are strong non zeros}
     \item{\code{zeros}}{ indexes of variables that are strong zeros}
   }}
   \item{\code{parameters}}{ a list of hyper-parameters for the method: 
   \describe{
     \item{\code{inter_len}}{ lengths of intervals}
     \item{\code{sel_prop}}{ if \code{adaptive = TRUE}, fraction of the 
     coefficients which are considered as strong zeros or strong non zeros}
   }}
   \item{\code{rSIR}}{ same as the input \code{object}}
   \item{\code{fit}}{ a list for LASSO fit with:
   \describe{
     \item{\code{glmnet}}{ result of the \code{\link[glmnet]{glmnet}} 
     function}
     \item{\code{lambda}}{ value of the best Lasso parameter by CV}
     \item{\code{x}}{ exploratory variable values as passed to fit the 
     model}
   }}
 }
 
 @details Different quality criteria used to select the best models among a
 list of models with different interval definitions. Quality criteria are:
 log-likelihood (\code{loglik}), cross-validation error as provided by the
 function \code{\link[glmnet]{glmnet}}, two versions of the AIC (\code{AIC} 
 and \code{AIC2}) and of the BIC (\code{BIC} and \code{BIC2}) in which the 
 number of parameters is either the number of non null intervals or the 
 number of non null parameters with respect to the original variables.
}
\description{
\code{sparseSIR} performs the second step of the method (shrinkage of ridge
SIR results
}
\examples{
set.seed(1140)
tsteps <- seq(0, 1, length = 200)
nsim <- 100
simulate_bm <- function() return(c(0, cumsum(rnorm(length(tsteps)-1, sd=1))))
x <- t(replicate(nsim, simulate_bm()))
beta <- cbind(sin(tsteps*3*pi/2), sin(tsteps*5*pi/2))
beta[((tsteps < 0.2) | (tsteps > 0.5)), 1] <- 0
beta[((tsteps < 0.6) | (tsteps > 0.75)), 2] <- 0
y <- log(abs(x \%*\% beta[ ,1]) + 1) + sqrt(abs(x \%*\% beta[ ,2]))
y <- y + rnorm(nsim, sd = 0.1)
res_ridge <- ridgeSIR(x, y, H = 10, d = 2, mu2 = 10^8)
res_sparse <- sparseSIR(res_ridge, rep(10, 20))

}
\references{
{Picheny, V., Servien, R., and Villa-Vialaneix, N. (2019) 
Interpretable sparse SIR for digitized functional data. 
\emph{Statistics and Computing}, \strong{29}(2), 255--267.}
}
\seealso{
\code{\link{ridgeSIR}}, \code{\link{project.sparseRes}}, 
\code{\link{SISIR}}
}
\author{
{Victor Picheny, \email{victor.picheny@inrae.fr}\cr
Remi Servien, \email{remi.servien@inrae.fr}\cr
Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ridgeSIR.R
\name{ridgeRes}
\alias{ridgeRes}
\alias{summary.ridgeRes}
\alias{print.ridgeRes}
\alias{ridgeRes-class}
\title{Print ridgeRes object}
\usage{
\method{summary}{ridgeRes}(object, ...)

\method{print}{ridgeRes}(x, ...)
}
\arguments{
\item{object}{a \code{ridgeRes} object}

\item{...}{not used}

\item{x}{a \code{ridgeRes} object}
}
\description{
Print a summary of the result of \code{\link{ridgeSIR}} (
\code{ridgeRes} object)
}
\seealso{
\code{\link{ridgeSIR}}
}
\author{
{Victor Picheny, \email{victor.picheny@inrae.fr}\cr 
Remi Servien, \email{remi.servien@inrae.fr}\cr
Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sparseSIR.R
\name{SISIRres}
\alias{SISIRres}
\alias{summary.SISIRres}
\alias{print.SISIRres}
\title{Print SISIRres object}
\usage{
\method{summary}{SISIRres}(object, ...)

\method{print}{SISIRres}(x, ...)
}
\arguments{
\item{object}{a \code{SISIRres} object}

\item{...}{not used}

\item{x}{a \code{SISIRres} object}
}
\description{
Print a summary of the result of \code{\link{SISIRres}} (
\code{SISIRres} object)
}
\seealso{
\code{\link{SISIR}}
}
\author{
{Victor Picheny, \email{victor.picheny@inrae.fr}\cr
Remi Servien, \email{remi.servien@inrae.fr}\cr
Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ridgeSIR.R
\name{tune.ridgeSIR}
\alias{tune.ridgeSIR}
\title{Cross-Validation for ridge SIR}
\usage{
tune.ridgeSIR(
  x,
  y,
  listH,
  list_mu2,
  list_d,
  nfolds = 10,
  parallel = TRUE,
  ncores = NULL
)
}
\arguments{
\item{x}{explanatory variables (numeric matrix or data frame)}

\item{y}{target variable (numeric vector)}

\item{listH}{list of the number of slices to be tested (numeric vector)}

\item{list_mu2}{list of ridge regularization parameters to be tested 
(numeric vector)}

\item{list_d}{list of the dimensions to be tested (numeric vector)}

\item{nfolds}{number of folds for the cross validation. Default is 10}

\item{parallel}{whether the computation should be performed in parallel or
not. Logical. Default is FALSE}

\item{ncores}{number of cores to use if \code{parallel = TRUE}. If left to 
NULL, all available cores minus one are used}
}
\value{
a data frame with tested parameters and corresponding CV error and 
estimation of R(d)
}
\description{
\code{tune.ridgeSIR} performs a Cross Validation for ridge SIR estimation
}
\examples{
set.seed(1115)
tsteps <- seq(0, 1, length = 200)
nsim <- 100
simulate_bm <- function() return(c(0, cumsum(rnorm(length(tsteps)-1, sd=1))))
x <- t(replicate(nsim, simulate_bm()))
beta <- cbind(sin(tsteps*3*pi/2), sin(tsteps*5*pi/2))
y <- log(abs(x \%*\% beta[ ,1])) + sqrt(abs(x \%*\% beta[ ,2]))
y <- y + rnorm(nsim, sd = 0.1)
list_mu2 <- 10^(0:10)
listH <- c(5, 10)
list_d <- 1:4
set.seed(1129)
\donttest{
res_tune <- tune.ridgeSIR(x, y, listH, list_mu2, list_d, nfolds = 10, 
                          parallel = TRUE, ncores = 2)}

}
\references{
{Picheny, V., Servien, R. and Villa-Vialaneix, N. (2016) 
Interpretable sparse SIR for digitized functional data.
\emph{Statistics and Computing}, \strong{29}(2), 255--267.}
}
\seealso{
\code{\link{ridgeSIR}}
}
\author{
{Victor Picheny, \email{victor.picheny@inrae.fr}\cr
Remi Servien, \email{remi.servien@inrae.fr}\cr
Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ridgeSIR.R
\name{ridgeSIR}
\alias{ridgeSIR}
\title{ridge SIR}
\usage{
ridgeSIR(x, y, H, d, mu2 = NULL)
}
\arguments{
\item{x}{explanatory variables (numeric matrix or data frame)}

\item{y}{target variable (numeric vector)}

\item{H}{number of slices (integer)}

\item{d}{number of dimensions to be kept}

\item{mu2}{ridge regularization parameter (numeric, positive)}
}
\value{
S3 object of class \code{ridgeRes}: a list consisting of
\describe{
   \item{\code{EDR}}{ the estimated EDR space (a p x d matrix)}
   \item{\code{condC}}{ the estimated slice projection on EDR (a d x H 
   matrix)}
   \item{\code{eigenvalues}}{ the eigenvalues obtained during the generalized 
   eigendecomposition performed by SIR}
   \item{\code{parameters}}{ a list of hyper-parameters for the method: 
   \describe{
     \item{\code{H}}{ number of slices}
     \item{\code{d}}{ dimension of the EDR space}
     \item{\code{mu2}}{ regularization parameter for the ridge penalty}
   }}
   \item{\code{utils}}{ useful outputs for further computations:
   \describe{
     \item{\code{Sigma}}{ covariance matrix for x}
     \item{\code{slices}}{ slice number for all observations}
     \item{\code{invsqrtS}}{ value of the inverse square root of the 
     regularized covariance matrix for x}
   }}
 }
}
\description{
\code{ridgeSIR} performs the first step of the method (ridge regularization
of SIR)
}
\details{
SI-SIR
}
\examples{
set.seed(1140)
tsteps <- seq(0, 1, length = 50)
simulate_bm <- function() return(c(0, cumsum(rnorm(length(tsteps)-1, sd=1))))
x <- t(replicate(50, simulate_bm()))
beta <- cbind(sin(tsteps*3*pi/2), sin(tsteps*5*pi/2)) 
y <- log(abs(x \%*\% beta[ ,1])) + sqrt(abs(x \%*\% beta[ ,2]))
y <- y + rnorm(50, sd = 0.1)
res_ridge <- ridgeSIR(x, y, H = 10, d = 2, mu2 = 10^8)
res_ridge

}
\references{
Picheny, V., Servien, R. and Villa-Vialaneix, N. (2019) 
Interpretable sparse SIR for digitized functional data. 
\emph{Statistics and Computing}, \strong{29}(2), 255--267.
}
\seealso{
\code{\link{sparseSIR}}, \code{\link{SISIR}}, 
\code{\link{tune.ridgeSIR}}
}
\author{
Victor Picheny, \email{victor.picheny@inrae.fr}\cr
Remi Servien, \email{remi.servien@inrae.fr}\cr
Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sparseSIR.R
\name{SISIR}
\alias{SISIR}
\title{Interval Sparse SIR}
\usage{
SISIR(
  object,
  inter_len = rep(1, nrow(object$EDR)),
  sel_prop = 0.05,
  itermax = Inf,
  minint = 2,
  parallel = TRUE,
  ncores = NULL
)
}
\arguments{
\item{object}{an object of class \code{ridgeRes} as obtained from the 
function \code{\link{ridgeSIR}}}

\item{inter_len}{(numeric) vector with interval lengths for the initial 
state. Default is to set one interval for each variable (all intervals have
length 1)}

\item{sel_prop}{fraction of the coefficients that will be considered as 
strong zeros and strong non zeros. Default to 0.05}

\item{itermax}{maximum number of iterations. Default to Inf}

\item{minint}{minimum number of intervals. Default to 2}

\item{parallel}{whether the computation should be performed in parallel or
not. Logical. Default is FALSE}

\item{ncores}{number of cores to use if \code{parallel = TRUE}. If left to 
NULL, all available cores minus one are used}
}
\value{
S3 object of class \code{SISIR}: a list consisting of
\describe{
   \item{\code{sEDR}}{ the estimated EDR spaces (a list of p x d matrices)}
   \item{\code{alpha}}{ the estimated shrinkage coefficients (a list of 
   vectors)}
   \item{\code{intervals}}{ the interval lengths (a list of vectors)}
   \item{\code{quality}}{ a data frame with various qualities for the model.
   The chosen quality measures are the same than for the function 
   \code{\link{sparseSIR}} plus the number of intervals \code{nbint}}
   \item{\code{init_sel_prop}}{ initial fraction of the coefficients which 
   are considered as strong zeros or strong non zeros}
   \item{\code{rSIR}}{ same as the input \code{object}}
 }
}
\description{
\code{SISIR} performs an automatic search of relevant intervals
}
\details{
Different quality criteria used to select the best models among a list of 
models with different interval definitions. Quality criteria are: 
log-likelihood (\code{loglik}), cross-validation error as provided by the
function \code{\link[glmnet]{glmnet}}, two versions of the AIC (\code{AIC} 
and \code{AIC2}) and of the BIC (\code{BIC} and \code{BIC2}) in which the 
number of parameters is either the number of non null intervals or the 
number of non null parameters with respect to the original variables
}
\examples{
set.seed(1140)
tsteps <- seq(0, 1, length = 200)
nsim <- 100
simulate_bm <- function() return(c(0, cumsum(rnorm(length(tsteps)-1, sd=1))))
x <- t(replicate(nsim, simulate_bm()))
beta <- cbind(sin(tsteps*3*pi/2), sin(tsteps*5*pi/2))
beta[((tsteps < 0.2) | (tsteps > 0.5)), 1] <- 0
beta[((tsteps < 0.6) | (tsteps > 0.75)), 2] <- 0
y <- log(abs(x \%*\% beta[ ,1]) + 1) + sqrt(abs(x \%*\% beta[ ,2]))
y <- y + rnorm(nsim, sd = 0.1)
res_ridge <- ridgeSIR(x, y, H = 10, d = 2, mu2 = 10^8)
\donttest{res_fused <- SISIR(res_ridge, rep(1, ncol(x)), ncores = 2)
res_fused}

}
\references{
Picheny, V., Servien, R. and Villa-Vialaneix, N. (2016) 
Interpretable sparse SIR for digitized functional data.
\emph{Statistics and Computing}, \strong{29}(2), 255--267.
}
\seealso{
\code{\link{ridgeSIR}}, \code{\link{sparseSIR}}
}
\author{
Victor Picheny, \email{victor.picheny@inrae.fr}\cr
Remi Servien, \email{remi.servien@inrae.fr}\cr
Nathalie Vialaneix, \email{nathalie.vialaneix@inrae.fr}
}
